from django_filters import rest_framework as filters

from product.models.category import ProductCategory
from product.models.product import Product


class ProductFilter(filters.FilterSet):
    min_final_price = filters.NumberFilter(
        field_name="final_price_amount",
        lookup_expr="gte",
        label="Minimum Final Price",
    )
    max_final_price = filters.NumberFilter(
        field_name="final_price_amount",
        lookup_expr="lte",
        label="Maximum Final Price",
    )
    category = filters.CharFilter(
        field_name="category__id", method="filter_category", label="Category"
    )
    min_discount = filters.NumberFilter(
        field_name="discount_value_amount",
        lookup_expr="gte",
        label="Minimum Discount Value",
    )
    max_discount = filters.NumberFilter(
        field_name="discount_value_amount",
        lookup_expr="lte",
        label="Maximum Discount Value",
    )
    min_review_average = filters.NumberFilter(
        field_name="review_average_field",
        lookup_expr="gte",
        label="Minimum Review Average",
    )
    min_likes = filters.NumberFilter(
        field_name="likes_count_field",
        lookup_expr="gte",
        label="Minimum Likes Count",
    )

    class Meta:
        model = Product
        fields = [
            "active",
            "min_final_price",
            "max_final_price",
            "category",
            "min_discount",
            "max_discount",
            "min_review_average",
            "min_likes",
        ]

    def filter_category(self, queryset, name, value):
        category_ids = value.split("_")
        all_relevant_category_ids = []

        for category_id in category_ids:
            try:
                category = ProductCategory.objects.get(id=category_id)
                descendant_ids = category.get_descendants(
                    include_self=True
                ).values_list("id", flat=True)
                all_relevant_category_ids.extend(descendant_ids)
            except ProductCategory.DoesNotExist:
                pass

        return queryset.filter(category__id__in=all_relevant_category_ids)



from django.utils.translation import gettext_lazy as _
from rest_framework.permissions import BasePermission


class IsSelfOrAdmin(BasePermission):
    message = _("You do not have permission to access this account.")

    def has_permission(self, request, view):
        return request.user and request.user.is_authenticated

    def has_object_permission(self, request, view, obj):
        return request.user.is_staff or obj.id == request.user.id


class IsOwnerOrReadOnly(BasePermission):
    message = _("You can only modify your own content.")

    def has_object_permission(self, request, view, obj):
        if request.method in ["GET", "HEAD", "OPTIONS"]:
            return True
        return hasattr(obj, "user") and obj.user == request.user


class IsStaffOrReadOnly(BasePermission):
    message = _("Only staff members can perform this action.")

    def has_permission(self, request, view):
        if request.method in ["GET", "HEAD", "OPTIONS"]:
            return True
        return request.user and request.user.is_staff


class IsAuthenticatedOrReadOnly(BasePermission):
    message = _("Authentication required for this action.")

    def has_permission(self, request, view):
        if request.method in ["GET", "HEAD", "OPTIONS"]:
            return True
        return request.user and request.user.is_authenticated



import factory
from django.contrib.contenttypes.models import ContentType

from core.helpers.factory import get_or_create_instance
from tag.models.tagged_item import TaggedItem


def get_or_create_product():
    return get_or_create_instance(
        app_label="product",
        model_name="Product",
        factory_module_path="product.factories.product",
        factory_class_name="ProductFactory",
    )


def get_or_create_tag():
    return get_or_create_instance(
        app_label="tag",
        model_name="Tag",
        factory_module_path="tag.factories.tag",
        factory_class_name="TagFactory",
    )


class TaggedItemFactory(factory.django.DjangoModelFactory):
    object_id = factory.SelfAttribute("content_object.id")
    content_type = factory.LazyAttribute(
        lambda o: ContentType.objects.get_for_model(o.content_object)
    )

    class Meta:
        exclude = ["content_object"]
        abstract = True


class TaggedProductFactory(TaggedItemFactory):
    content_object = factory.LazyFunction(get_or_create_product)
    tag = factory.LazyFunction(get_or_create_tag)

    class Meta:
        model = TaggedItem



from rest_framework import pagination
from rest_framework.response import Response


class PageNumberPaginator(pagination.PageNumberPagination):
    page_size_query_param = "page_size"
    max_page_size = 100

    def get_paginated_response(self, data):
        return Response(
            {
                "links": {
                    "next": self.get_next_link(),
                    "previous": self.get_previous_link(),
                },
                "count": self.page.paginator.count,
                "total_pages": self.page.paginator.num_pages,
                "page_size": self.page.paginator.per_page,
                "page_total_results": len(data),
                "page": self.page.number,
                "results": data,
            }
        )



import json
import re
from collections.abc import Mapping
from typing import Any, NotRequired, TypedDict

from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.utils.functional import Promise
from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import extend_schema
from parler_rest.fields import TranslatedFieldsField
from rest_framework import serializers as rf_serializers


class TranslatedFieldExtended(TranslatedFieldsField):
    def to_internal_value(self, data):
        if data is None:
            return {}
        if isinstance(data, str):
            data = json.loads(data)
        if not isinstance(data, dict):
            self.fail("invalid")
        if not self.allow_empty and len(data) == 0:
            self.fail("empty")

        result, errors = {}, {}
        for lang_code, model_fields in data.items():
            serializer = self.serializer_class(data=model_fields)
            if serializer.is_valid():
                result[lang_code] = serializer.validated_data
            else:
                errors[lang_code] = serializer.errors

        if errors:
            raise rf_serializers.ValidationError(errors)
        return result


def camel_to_words(name):
    result = re.sub(r"(?<!^)(?=[A-Z])", " ", name)
    return result


class DisplayConfig(TypedDict):
    tag: NotRequired[str]
    display_name: NotRequired[str]
    display_name_plural: NotRequired[str]


def create_schema_view_config(
    model_class: type[models.Model],
    serializers: dict | None = None,
    error_serializer=None,
    additional_responses=None,
    display_config: DisplayConfig | Mapping[str, Any] | None = None,
):
    """
    Create schema configuration for a ViewSet with translation support.

    Args:
        model_class: The Django model class
        serializers: Dict with keys: list_serializer, detail_serializer, write_serializer
        error_serializer: Serializer for error responses
        additional_responses: Dict with additional responses per action
        display_config: DisplayConfig or dict with optional keys:
            - tag: Override for OpenAPI tag (default: model's verbose_name_plural)
            - display_name: Override for singular name (default: model's verbose_name)
            - display_name_plural: Override for plural name (default: model's verbose_name_plural)

    Note:
        This function fully supports Django translations. If your model uses
        gettext_lazy for verbose_name/verbose_name_plural, all generated
        summaries and descriptions will be properly translated.
    """
    if serializers is None:
        serializers = {}
    if additional_responses is None:
        additional_responses = {}
    if display_config is None:
        display_config = {}

    model_name = model_class.__name__

    display_name = display_config.get("display_name")
    if display_name is None:
        if hasattr(model_class._meta, "verbose_name"):
            display_name = model_class._meta.verbose_name
        else:
            display_name = camel_to_words(model_name).lower()

    display_name_plural = display_config.get("display_name_plural")
    if display_name_plural is None:
        if hasattr(model_class._meta, "verbose_name_plural"):
            display_name_plural = model_class._meta.verbose_name_plural
        elif isinstance(display_name, Promise):
            display_name_plural = display_name
        else:
            display_name_plural = f"{display_name}s"

    tag = display_config.get("tag")
    if tag is None:
        if hasattr(model_class._meta, "verbose_name_plural"):
            tag = model_class._meta.verbose_name_plural
        else:
            tag = camel_to_words(model_name)

    list_serializer = serializers.get("list_serializer")
    detail_serializer = serializers.get("detail_serializer")
    write_serializer = serializers.get("write_serializer")

    base_error_responses = (
        {
            400: error_serializer,
            401: error_serializer,
            403: error_serializer,
            404: error_serializer,
            500: error_serializer,
        }
        if error_serializer
        else {}
    )

    config = {
        "list": extend_schema(
            operation_id=f"list{model_name}",
            summary=_("List %(name)s") % {"name": display_name_plural},
            description=_(
                "Retrieve a list of %(name)s with filtering and search capabilities."
            )
            % {"name": display_name_plural},
            tags=[tag],
            responses={
                200: list_serializer(many=True) if list_serializer else None,
                **base_error_responses,
                **additional_responses.get("list", {}),
            },
        ),
        "create": extend_schema(
            operation_id=f"create{model_name}",
            summary=_("Create a %(name)s") % {"name": display_name},
            description=_("Create a new %(name)s. Requires authentication.")
            % {"name": display_name},
            tags=[tag],
            request=write_serializer,
            responses={
                201: detail_serializer,
                **base_error_responses,
                **additional_responses.get("create", {}),
            },
        ),
        "retrieve": extend_schema(
            operation_id=f"retrieve{model_name}",
            summary=_("Retrieve a %(name)s") % {"name": display_name},
            description=_("Get detailed information about a specific %(name)s.")
            % {"name": display_name},
            tags=[tag],
            responses={
                200: detail_serializer,
                **{k: v for k, v in base_error_responses.items() if k != 400},
                **additional_responses.get("retrieve", {}),
            },
        ),
        "update": extend_schema(
            operation_id=f"update{model_name}",
            summary=_("Update a %(name)s") % {"name": display_name},
            description=_(
                "Update %(name)s information. Requires authentication."
            )
            % {"name": display_name},
            tags=[tag],
            request=write_serializer,
            responses={
                200: detail_serializer,
                **base_error_responses,
                **additional_responses.get("update", {}),
            },
        ),
        "partial_update": extend_schema(
            operation_id=f"partialUpdate{model_name}",
            summary=_("Partially update a %(name)s") % {"name": display_name},
            description=_(
                "Partially update %(name)s information. Requires authentication."
            )
            % {"name": display_name},
            tags=[tag],
            request=write_serializer,
            responses={
                200: detail_serializer,
                **base_error_responses,
                **additional_responses.get("partial_update", {}),
            },
        ),
        "destroy": extend_schema(
            operation_id=f"destroy{model_name}",
            summary=_("Delete a %(name)s") % {"name": display_name},
            description=_("Delete a %(name)s. Requires authentication.")
            % {"name": display_name},
            tags=[tag],
            responses={
                204: None,
                **{
                    k: v
                    for k, v in base_error_responses.items()
                    if k not in [400, 500]
                },
                **additional_responses.get("destroy", {}),
            },
        ),
    }

    return config


class MultiSerializerMixin:
    action = None
    serializers = {
        "default": None,
        "list": None,
        "create": None,
        "retrieve": None,
        "update": None,
        "partial_update": None,
        "destroy": None,
    }
    request_serializers = {}
    response_serializers = {}

    def get_serializer_class(self):
        has_explicit_serializer_class = (
            hasattr(self, "serializer_class")
            and self.serializer_class is not None
        )

        if hasattr(self, "serializers") and has_explicit_serializer_class:
            raise ImproperlyConfigured(
                "{cls} should only define either `serializer_class` or "
                "`serializers`.".format(cls=self.__class__.__name__)
            )

        if not hasattr(self, "serializers") or self.serializers is None:
            raise ImproperlyConfigured(
                "{cls} is missing the serializers attribute. Define "
                "{cls}.serializers, or override "
                "{cls}.get_serializer_class().".format(
                    cls=self.__class__.__name__
                )
            )

        serializer_class = self.serializers.get(self.action)
        if serializer_class is None:
            serializer_class = self.serializers.get("default")

        if serializer_class is None:
            raise ImproperlyConfigured(
                "No serializer found for action '{action}' and no default serializer defined. "
                "Define {cls}.serializers['{action}'] or {cls}.serializers['default'], "
                "or override {cls}.get_serializer_class().".format(
                    action=self.action, cls=self.__class__.__name__
                )
            )

        return serializer_class

    def get_request_serializer_class(self):
        if hasattr(self, "request_serializers") and self.request_serializers:
            request_serializer = self.request_serializers.get(self.action)
            if request_serializer is not None:
                return request_serializer

        return self.get_serializer_class()

    def get_response_serializer_class(self):
        if hasattr(self, "response_serializers") and self.response_serializers:
            response_serializer = self.response_serializers.get(self.action)
            if response_serializer is not None:
                return response_serializer

        return self.get_serializer_class()

    def get_serializer_for_schema(self, action_name=None):
        if action_name is None:
            action_name = self.action

        original_action = self.action
        self.action = action_name

        try:
            request_serializer = self.get_request_serializer_class()
            response_serializer = self.get_response_serializer_class()
            return {
                "request": request_serializer,
                "response": response_serializer,
            }
        finally:
            self.action = original_action

    def get_serializer_context(self):
        context = (
            super().get_serializer_context()
            if hasattr(super(), "get_serializer_context")
            else {}
        )
        context.update(
            {
                "action": self.action,
                "view": self,
            }
        )
        return context



import django_filters
from django.utils.translation import gettext_lazy as _

from order.models.item import OrderItem
from order.models.order import Order


class OrderFilter(django_filters.FilterSet):
    id = django_filters.NumberFilter(field_name="id")
    user = django_filters.NumberFilter(field_name="user")
    user_id = django_filters.NumberFilter(field_name="user_id")
    country = django_filters.NumberFilter(field_name="country")
    region = django_filters.NumberFilter(field_name="region")
    pay_way = django_filters.NumberFilter(field_name="pay_way")
    status = django_filters.CharFilter(field_name="status")
    payment_status = django_filters.CharFilter(field_name="payment_status")
    document_type = django_filters.CharFilter(field_name="document_type")
    floor = django_filters.CharFilter(field_name="floor")
    location_type = django_filters.CharFilter(field_name="location_type")

    created_after = django_filters.DateTimeFilter(
        field_name="created_at", lookup_expr="gte"
    )
    created_before = django_filters.DateTimeFilter(
        field_name="created_at", lookup_expr="lte"
    )
    status_updated_after = django_filters.DateTimeFilter(
        field_name="status_updated_at", lookup_expr="gte"
    )
    status_updated_before = django_filters.DateTimeFilter(
        field_name="status_updated_at", lookup_expr="lte"
    )

    paid_amount_min = django_filters.NumberFilter(
        field_name="paid_amount", lookup_expr="gte"
    )
    paid_amount_max = django_filters.NumberFilter(
        field_name="paid_amount", lookup_expr="lte"
    )
    shipping_price_min = django_filters.NumberFilter(
        field_name="shipping_price", lookup_expr="gte"
    )
    shipping_price_max = django_filters.NumberFilter(
        field_name="shipping_price", lookup_expr="lte"
    )

    has_user = django_filters.BooleanFilter(method="filter_has_user")
    has_tracking = django_filters.BooleanFilter(method="filter_has_tracking")
    has_payment_id = django_filters.BooleanFilter(
        method="filter_has_payment_id"
    )
    has_mobile_phone = django_filters.BooleanFilter(
        method="filter_has_mobile_phone"
    )
    has_customer_notes = django_filters.BooleanFilter(
        method="filter_has_customer_notes"
    )

    city = django_filters.CharFilter(field_name="city", lookup_expr="icontains")
    zipcode = django_filters.CharFilter(field_name="zipcode")
    place = django_filters.CharFilter(
        field_name="place", lookup_expr="icontains"
    )

    class Meta:
        model = Order
        fields = [
            "id",
            "user",
            "user_id",
            "country",
            "region",
            "pay_way",
            "status",
            "payment_status",
            "document_type",
            "floor",
            "location_type",
            "city",
            "zipcode",
            "place",
        ]

    def filter_has_user(self, queryset, name, value):
        if value:
            return queryset.filter(user__isnull=False)
        return queryset.filter(user__isnull=True)

    def filter_has_tracking(self, queryset, name, value):
        if value:
            return queryset.exclude(tracking_number="")
        return queryset.filter(tracking_number="")

    def filter_has_payment_id(self, queryset, name, value):
        if value:
            return queryset.exclude(payment_id="")
        return queryset.filter(payment_id="")

    def filter_has_mobile_phone(self, queryset, name, value):
        if value:
            return queryset.filter(mobile_phone__isnull=False)
        return queryset.filter(mobile_phone__isnull=True)

    def filter_has_customer_notes(self, queryset, name, value):
        if value:
            return queryset.exclude(customer_notes="")
        return queryset.filter(customer_notes="")


class OrderItemFilter(django_filters.FilterSet):
    id = django_filters.NumberFilter(
        field_name="id",
        lookup_expr="exact",
        help_text=_("Filter by order item ID"),
    )
    order = django_filters.NumberFilter(
        field_name="order__id",
        lookup_expr="exact",
        help_text=_("Filter by order ID"),
    )
    product = django_filters.NumberFilter(
        field_name="product__id",
        lookup_expr="exact",
        help_text=_("Filter by product ID"),
    )
    product_name = django_filters.CharFilter(
        field_name="product__translations__name",
        lookup_expr="icontains",
        help_text=_("Filter by product name (partial match)"),
    )

    price_min = django_filters.NumberFilter(
        field_name="price",
        lookup_expr="gte",
        help_text=_("Filter by minimum price"),
    )
    price_max = django_filters.NumberFilter(
        field_name="price",
        lookup_expr="lte",
        help_text=_("Filter by maximum price"),
    )

    quantity_min = django_filters.NumberFilter(
        field_name="quantity",
        lookup_expr="gte",
        help_text=_("Filter by minimum quantity"),
    )
    quantity_max = django_filters.NumberFilter(
        field_name="quantity",
        lookup_expr="lte",
        help_text=_("Filter by maximum quantity"),
    )

    is_refunded = django_filters.BooleanFilter(
        field_name="is_refunded",
        help_text=_("Filter by refund status"),
    )
    has_notes = django_filters.BooleanFilter(
        method="filter_has_notes",
        help_text=_("Filter items that have/don't have notes"),
    )

    created_after = django_filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="gte",
        help_text=_("Filter items created after this date"),
    )
    created_before = django_filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="lte",
        help_text=_("Filter items created before this date"),
    )

    class Meta:
        model = OrderItem
        fields = [
            "id",
            "order",
            "product",
            "product_name",
            "price_min",
            "price_max",
            "quantity_min",
            "quantity_max",
            "is_refunded",
            "has_notes",
            "created_after",
            "created_before",
        ]

    def filter_has_notes(self, queryset, name, value):
        if value is True:
            return queryset.exclude(notes="")
        elif value is False:
            return queryset.filter(notes="")
        return queryset



import importlib
import os
import pkgutil
import time
import traceback
from types import ModuleType
from typing import TypeVar, cast

import factory
from django.apps import apps
from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import IntegrityError, models

DEFAULT_COUNT = settings.SEED_DEFAULT_COUNT
BATCH_SIZE = settings.SEED_BATCH_SIZE

F = TypeVar("F", bound=factory.django.DjangoModelFactory)


class Command(BaseCommand):
    help = "Seed all models with their factories"

    def add_arguments(self, parser):
        parser.add_argument(
            "--count",
            type=int,
            default=DEFAULT_COUNT,
            help=f"Number of records to create for each model (default: {DEFAULT_COUNT})",
        )
        parser.add_argument(
            "--model-counts",
            type=str,
            help="Comma-separated list of model-specific counts in the form 'ModelName1=10,ModelName2=100'",
        )

    def handle(self, *args, **options):
        self.stdout.write(
            self.style.NOTICE("Starting the seeding process...\n")
        )
        start_total_time = time.time()
        count = options["count"]
        model_counts = self.parse_model_counts(options["model_counts"])
        available_models = self.get_available_models()
        success_messages = []
        factory_timings: dict[str, float] = {}

        initial_counts = self.get_initial_counts()

        for model_name in model_counts:
            if model_name not in available_models:
                self.stdout.write(
                    self.style.WARNING(
                        f"Warning: Model '{model_name}' does not exist. "
                        f"Available models are: {', '.join(available_models)}"
                    )
                )

        factory_modules = self.find_factory_modules()

        created_counts: dict[str, int] = {}

        for module_path in factory_modules:
            if isinstance(module_path, str):
                result = self.process_module(
                    module_path,
                    model_counts,
                    count,
                    created_counts,
                    factory_timings,
                )
                if result:
                    success_messages.extend(result)

        for message in success_messages:
            self.stdout.write(message)

        total_time = time.time() - start_total_time
        self.stdout.write(self.style.NOTICE("\nSeeding process completed.\n"))

        self.stdout.write(self.style.NOTICE("Created instances summary:"))
        for factory_class, count in created_counts.items():
            timing = factory_timings.get(factory_class, 0)
            self.stdout.write(
                self.style.SUCCESS(
                    f"{factory_class:<30} : {count:<5} ({timing:.2f}s)"
                )
            )

        self.print_created_counts(initial_counts)

        self.stdout.write(
            self.style.NOTICE(f"\nTotal seeding time: {total_time:.2f} seconds")
        )

    def process_module(
        self,
        module_path: str,
        model_counts: dict[str, int],
        default_count: int,
        created_counts: dict[str, int],
        factory_timings: dict[str, float],
    ):
        success_messages = []
        module = importlib.import_module(module_path)
        factory_classes = self.get_factory_classes(module)
        if not factory_classes:
            return success_messages

        self.stdout.write(
            self.style.NOTICE(f"\nProcessing module: {module_path}")
        )
        try:
            for factory_class in factory_classes:
                model_name = factory_class._meta.model.__name__
                if model_name.endswith("Translation"):
                    continue
                model_count = model_counts.get(model_name, default_count)

                self.stdout.write(
                    self.style.NOTICE(
                        f"Running factory: {factory_class.__name__} for {model_name}"
                    )
                )

                start_time = time.time()
                self.create_records(factory_class, model_count, created_counts)
                elapsed_time = time.time() - start_time

                factory_timings[factory_class.__name__] = elapsed_time

        except Exception as e:
            tb = traceback.format_exc()
            error_message = f"Failed to seed using {module_path}: {e!s}\n{tb}"
            self.stdout.write(self.style.ERROR(error_message))
            success_messages.append(self.style.ERROR(error_message))

        return success_messages

    def create_records(
        self, factory_class: type[F], count: int, created_counts: dict[str, int]
    ):
        factory_name = factory_class.__name__
        created_counts[factory_name] = created_counts.get(factory_name, 0)

        self.stdout.write(
            self.style.NOTICE(f"Creating {count} records using {factory_name}")
        )

        instances = []
        for i in range(count):
            try:
                instance = factory_class.create()
                self.save_related_objects(instance)
                instances.append(instance)

                if len(instances) >= BATCH_SIZE:
                    self.save_batch(instances, factory_class)
                    created_counts[factory_name] += len(instances)
                    instances.clear()

            except Exception as e:
                error_message = (
                    f"Failed to create {factory_class._meta.model.__name__} instance "
                    f"{i + 1}/{count}: {e!s}"
                )
                self.stdout.write(self.style.ERROR(error_message))

        if instances:
            self.save_batch(instances, factory_class)
            created_counts[factory_name] += len(instances)

        self.stdout.write(
            self.style.NOTICE(
                f"Completed creating {count} records using {factory_name}"
            )
        )

        return count

    @staticmethod
    def save_related_objects(instance: models.Model):
        for field in instance._meta.get_fields():
            if field.is_relation and field.many_to_one:
                related_instance = getattr(instance, field.name)
                if related_instance and not related_instance.pk:
                    related_instance.save()

    def save_batch(self, instances: list[models.Model], factory_class: type[F]):
        try:
            factory_class._meta.model.objects.bulk_create(
                instances, batch_size=BATCH_SIZE
            )
            self.stdout.write(
                self.style.NOTICE(f"Saved batch of {len(instances)} records.")
            )
        except IntegrityError:
            for instance in instances:
                try:
                    instance.save()
                except IntegrityError as e:
                    self.stdout.write(
                        self.style.ERROR(f"Failed to save instance: {e!s}")
                    )

    def parse_model_counts(self, model_counts_str: str | None):
        if not model_counts_str:
            return {}

        model_counts = {}
        pairs = model_counts_str.split(",")
        for pair in pairs:
            try:
                model_name, count = pair.split("=")
                model_counts[model_name.strip()] = int(count.strip())
            except ValueError:
                self.stdout.write(
                    self.style.ERROR(f"Invalid format for model count: {pair}")
                )
        return model_counts

    def find_factory_modules(self):
        factory_modules = []
        for app in apps.get_app_configs():
            app_path = app.module.__path__[0]
            for root, _, files in os.walk(app_path):
                for file in files:
                    if file == "factories.py":
                        module_path = self.path_to_module(root, file)
                        factory_modules.append(module_path)
                if "factories" in os.listdir(root):
                    factories_dir = os.path.join(root, "factories")
                    for _, name, is_pkg in pkgutil.iter_modules(
                        [factories_dir]
                    ):
                        if not is_pkg:
                            module_path = self.path_to_module(
                                factories_dir, f"{name}.py"
                            )
                            factory_modules.append(module_path)
        return factory_modules

    @staticmethod
    def path_to_module(root: str, file: str):
        module_path = os.path.relpath(os.path.join(root, file))
        module_path = module_path.replace(os.sep, ".")
        if module_path.endswith(".py"):
            module_path = module_path[:-3]
        return module_path

    @staticmethod
    def get_factory_classes(module: ModuleType):
        factory_classes = []
        for attr_name in dir(module):
            attr = getattr(module, attr_name)
            if hasattr(attr, "_meta") and hasattr(attr._meta, "model"):  # noqa
                if attr._meta.model is not None and "Factory" in attr.__name__:
                    factory_classes.append(cast("type[F]", attr))
        return factory_classes

    def get_available_models(self):
        available_models = []
        factory_modules = self.find_factory_modules()
        for module_path in factory_modules:
            module = importlib.import_module(module_path)
            factory_classes = self.get_factory_classes(module)
            for factory_class in factory_classes:
                model_name = factory_class._meta.model.__name__
                if not model_name.endswith("Translation"):
                    available_models.append(model_name)
        return available_models

    @staticmethod
    def get_initial_counts():
        initial_counts = {}
        for model in apps.get_models():
            model_name = model.__name__
            if model_name.endswith("Translation"):
                continue
            count = model.objects.count()
            initial_counts[model_name] = count
        return initial_counts

    def print_created_counts(self, initial_counts: dict[str, int]):
        self.stdout.write(
            self.style.NOTICE("\nTotal records created in the database:")
        )
        for model in apps.get_models():
            model_name = model.__name__
            if model_name.endswith("Translation"):
                continue
            initial_count = initial_counts.get(model_name, 0)
            current_count = model.objects.count()
            created_count = current_count - initial_count
            if created_count > 0:
                self.stdout.write(
                    self.style.SUCCESS(f"{model_name:<30} : {created_count:<5}")
                )



from django.db.models import Count
from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from blog.models.tag import BlogTag


class BlogTagFilter(filters.FilterSet):
    id = filters.NumberFilter(
        field_name="id",
        lookup_expr="exact",
        help_text=_("Filter by tag ID"),
    )
    active = filters.BooleanFilter(
        field_name="active",
        help_text=_("Filter by active status"),
    )
    name = filters.CharFilter(
        field_name="translations__name",
        lookup_expr="icontains",
        help_text=_("Filter by tag name (partial match)"),
    )

    min_posts = filters.NumberFilter(
        method="filter_min_posts",
        help_text=_("Filter tags with at least X posts"),
    )
    max_posts = filters.NumberFilter(
        method="filter_max_posts",
        help_text=_("Filter tags with at most X posts"),
    )
    has_posts = filters.BooleanFilter(
        method="filter_has_posts",
        help_text=_("Filter tags that have/don't have posts"),
    )

    class Meta:
        model = BlogTag
        fields = [
            "id",
            "active",
            "name",
            "min_posts",
            "max_posts",
            "has_posts",
        ]

    def filter_min_posts(self, queryset, name, value):
        if value is not None:
            return queryset.annotate(post_count=Count("blog_posts")).filter(
                post_count__gte=value
            )
        return queryset

    def filter_max_posts(self, queryset, name, value):
        if value is not None:
            return queryset.annotate(post_count=Count("blog_posts")).filter(
                post_count__lte=value
            )
        return queryset

    def filter_has_posts(self, queryset, name, value):
        if value is True:
            return queryset.annotate(post_count=Count("blog_posts")).filter(
                post_count__gt=0
            )
        elif value is False:
            return queryset.annotate(post_count=Count("blog_posts")).filter(
                post_count=0
            )
        return queryset



import base64
import hashlib
import os

from celery import Celery
from celery.exceptions import CeleryError
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from django.conf import settings
from django.db import DatabaseError, connection
from django.middleware.csrf import get_token
from django.shortcuts import redirect
from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import extend_schema
from redis import Redis, RedisError
from rest_framework import status
from rest_framework.decorators import action, api_view
from rest_framework.metadata import SimpleMetadata
from rest_framework.response import Response
from rest_framework.viewsets import ModelViewSet

from core.api.serializers import HealthCheckResponseSerializer
from core.pagination.cursor import CursorPaginator
from core.pagination.limit_offset import LimitOffsetPaginator
from core.pagination.page_number import PageNumberPaginator
from core.utils.views import TranslationsProcessingMixin

default_language = settings.PARLER_DEFAULT_LANGUAGE_CODE


class Metadata(SimpleMetadata):
    def determine_metadata(self, request, view):
        metadata = super().determine_metadata(request, view)
        metadata["filterset_fields"] = getattr(view, "filterset_fields", [])
        metadata["ordering_fields"] = getattr(view, "ordering_fields", [])
        metadata["ordering"] = getattr(view, "ordering", [])
        metadata["search_fields"] = getattr(view, "search_fields", [])
        return metadata


class PaginationModelViewSet(ModelViewSet):
    @property
    def paginator(self):
        if not hasattr(self, "_paginator"):
            if self.pagination_class is None:
                self._paginator = None
            else:
                pagination_type = self.request.query_params.get(
                    "pagination_type", ""
                ).lower()
                paginator_mapping = {
                    "page_number": PageNumberPaginator,
                    "limit_offset": LimitOffsetPaginator,
                    "cursor": CursorPaginator,
                }
                self._paginator = paginator_mapping.get(
                    pagination_type, self.pagination_class
                )()

        return self._paginator

    def paginate_and_serialize(
        self, queryset, request, many=True, serializer_class=None
    ):
        pagination_param = request.query_params.get(
            "pagination", "true"
        ).lower()

        if serializer_class is None:
            serializer_class = self.get_serializer_class()

        if pagination_param == "false":
            serializer = serializer_class(
                queryset, many=many, context=self.get_serializer_context()
            )
            return Response(serializer.data, status=status.HTTP_200_OK)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = serializer_class(
                page, many=many, context=self.get_serializer_context()
            )
            return self.get_paginated_response(serializer.data)

        serializer = serializer_class(
            queryset, many=many, context=self.get_serializer_context()
        )
        return Response(serializer.data, status=status.HTTP_200_OK)


class TranslationsModelViewSet(TranslationsProcessingMixin, ModelViewSet):
    def get_serializer_context(self):
        context = super().get_serializer_context()
        context["language"] = self.request.query_params.get(
            "language", default_language
        )
        return context

    def create(self, request, *args, **kwargs):
        request = self.process_translations_data(request)
        return super().create(request, *args, **kwargs)

    def update(self, request, *args, **kwargs):
        request = self.process_translations_data(request)
        return super().update(request, *args, **kwargs)

    def partial_update(self, request, *args, **kwargs):
        return super().partial_update(request, *args, **kwargs)


class BaseModelViewSet(TranslationsModelViewSet, PaginationModelViewSet):
    metadata_class = Metadata

    def get_response_serializer_class(self):
        if hasattr(super(), "get_response_serializer_class"):
            return super().get_response_serializer_class()
        return self.get_serializer_class()

    def get_request_serializer_class(self):
        if hasattr(super(), "get_request_serializer_class"):
            return super().get_request_serializer_class()
        return self.get_serializer_class()

    def create(self, request, *args, **kwargs):
        request_serializer_class = self.get_request_serializer_class()
        serializer = request_serializer_class(
            data=request.data, context=self.get_serializer_context()
        )
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)

        response_serializer_class = self.get_response_serializer_class()
        response_serializer = response_serializer_class(
            serializer.instance, context=self.get_serializer_context()
        )

        headers = self.get_success_headers(response_serializer.data)
        return Response(
            response_serializer.data,
            status=status.HTTP_201_CREATED,
            headers=headers,
        )

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop("partial", False)
        instance = self.get_object()

        request_serializer_class = self.get_request_serializer_class()
        serializer = request_serializer_class(
            instance,
            data=request.data,
            partial=partial,
            context=self.get_serializer_context(),
        )
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, "_prefetched_objects_cache", None):
            instance._prefetched_objects_cache = {}

        response_serializer_class = self.get_response_serializer_class()
        response_serializer = response_serializer_class(
            serializer.instance, context=self.get_serializer_context()
        )
        return Response(response_serializer.data)

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        response_serializer_class = self.get_response_serializer_class()
        serializer = response_serializer_class(
            instance, context=self.get_serializer_context()
        )
        return Response(serializer.data)

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        return self.paginate_and_serialize(
            queryset, request, serializer_class=self.get_serializer_class()
        )

    @action(detail=False, methods=["GET"])
    def api_schema(self, request, *args, **kwargs):
        meta = self.metadata_class()
        data = meta.determine_metadata(request, self)
        return Response(data)


@extend_schema(
    summary=_("Check the health status of database, Redis, and Celery"),
    description=_("Check the health status of database, Redis, and Celery"),
    tags=["Health"],
    responses=HealthCheckResponseSerializer,
)
@api_view(["GET"])
def health_check(request):
    health_status = {
        "database": False,
        "redis": False,
        "celery": False,
    }

    try:
        connection.cursor()
        health_status["database"] = True
    except DatabaseError:
        health_status["database"] = False

    try:
        redis_conn = Redis(host=settings.REDIS_HOST, port=settings.REDIS_PORT)
        redis_conn.ping()
        health_status["redis"] = True
    except RedisError:
        health_status["redis"] = False

    try:
        celery_app = Celery(broker=settings.CELERY_BROKER_URL)
        celery_status = celery_app.control.ping()
        health_status["celery"] = bool(celery_status)
    except CeleryError:
        health_status["celery"] = False

    response = Response(health_status)
    get_token(request)
    return response


def encrypt_token(token: str, secret_key: str) -> str:
    key = hashlib.sha256(secret_key.encode()).digest()
    nonce = os.urandom(16)
    cipher = Cipher(
        algorithms.AES(key), modes.GCM(nonce), backend=default_backend()
    )
    encryptor = cipher.encryptor()
    ciphertext = encryptor.update(token.encode()) + encryptor.finalize()
    encrypted_token = base64.urlsafe_b64encode(
        nonce + encryptor.tag + ciphertext
    ).decode("utf-8")
    return encrypted_token


def redirect_to_frontend(request, *args, **kwargs):
    from knox.models import get_token_model  # noqa: PLC0415

    AuthToken = get_token_model()
    user = request.user

    if user.is_authenticated:
        _, token = AuthToken.objects.create(user)
        encrypted_token = encrypt_token(token, settings.SECRET_KEY)
    else:
        encrypted_token = ""

    frontend_url = settings.NUXT_BASE_URL
    redirect_path = "/account/provider/callback"
    response = redirect(
        f"{frontend_url}{redirect_path}?encrypted_token={encrypted_token}"
    )

    response.headers["X-Encrypted-Token"] = encrypted_token

    return response



from __future__ import annotations

from django.conf import settings
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import extend_schema_view
from rest_framework.filters import SearchFilter

from blog.filters.tag import BlogTagFilter
from blog.models.tag import BlogTag
from blog.serializers.tag import (
    BlogTagDetailSerializer,
    BlogTagSerializer,
    BlogTagWriteSerializer,
)
from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from core.utils.views import cache_methods


@extend_schema_view(
    **create_schema_view_config(
        model_class=BlogTag,
        display_config={
            "tag": "Blog Tags",
        },
        serializers={
            "list_serializer": BlogTagSerializer,
            "detail_serializer": BlogTagDetailSerializer,
            "write_serializer": BlogTagWriteSerializer,
        },
        error_serializer=ErrorResponseSerializer,
        additional_responses={
            "create": {201: BlogTagDetailSerializer},
            "update": {200: BlogTagDetailSerializer},
            "partial_update": {200: BlogTagDetailSerializer},
        },
    )
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class BlogTagViewSet(MultiSerializerMixin, BaseModelViewSet):
    queryset = BlogTag.objects.all()
    serializers = {
        "default": BlogTagDetailSerializer,
        "list": BlogTagSerializer,
        "retrieve": BlogTagDetailSerializer,
        "create": BlogTagWriteSerializer,
        "update": BlogTagWriteSerializer,
        "partial_update": BlogTagWriteSerializer,
    }
    response_serializers = {
        "create": BlogTagDetailSerializer,
        "update": BlogTagDetailSerializer,
        "partial_update": BlogTagDetailSerializer,
    }
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_class = BlogTagFilter
    ordering_fields = [
        "id",
        "active",
        "created_at",
        "updated_at",
        "sort_order",
    ]
    ordering = ["-created_at"]
    search_fields = ["translations__name"]



from django.db.models import Q
from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from blog.models import BlogPost


class BlogPostFilter(filters.FilterSet):
    title = filters.CharFilter(
        field_name="translations__title",
        lookup_expr="icontains",
        help_text=_("Filter by title (case-insensitive)"),
    )
    author_email = filters.CharFilter(
        field_name="author__user__email",
        lookup_expr="icontains",
        help_text=_("Filter by author email (case-insensitive)"),
    )
    min_likes = filters.NumberFilter(
        field_name="likes_count_field",
        lookup_expr="gte",
        label="Minimum Likes Count",
        help_text=_("Filter by minimum number of likes"),
    )
    min_comments = filters.NumberFilter(
        field_name="comments_count_field",
        lookup_expr="gte",
        label="Minimum Comments Count",
        help_text=_("Filter by minimum number of comments"),
    )
    min_tags = filters.NumberFilter(
        field_name="tags_count_field",
        lookup_expr="gte",
        label="Minimum Tags Count",
        help_text=_("Filter by minimum number of tags"),
    )
    featured = filters.BooleanFilter(
        field_name="featured",
        lookup_expr="exact",
        help_text=_("Filter by featured status"),
    )
    is_published = filters.BooleanFilter(
        field_name="is_published",
        lookup_expr="exact",
        help_text=_("Filter by published status"),
    )
    min_view_count = filters.NumberFilter(
        field_name="view_count",
        lookup_expr="gte",
        label="Minimum View Count",
        help_text=_("Filter by minimum number of views"),
    )
    category_name = filters.CharFilter(
        field_name="category__translations__name",
        lookup_expr="icontains",
        help_text=_("Filter by category name (case-insensitive)"),
    )
    tag_name = filters.CharFilter(
        field_name="tags__translations__label",
        lookup_expr="icontains",
        help_text=_("Filter by tag label (case-insensitive)"),
    )
    author_name = filters.CharFilter(
        method="filter_author_name",
        help_text=_("Filter by author full name (case-insensitive)"),
    )

    class Meta:
        model = BlogPost
        fields = [
            "id",
            "tags",
            "slug",
            "author",
            "category",
            "title",
            "author_email",
            "author_name",
            "category_name",
            "tag_name",
            "featured",
            "is_published",
            "min_likes",
            "min_comments",
            "min_tags",
            "min_view_count",
        ]

    def filter_author_name(self, queryset, name, value):
        if value:
            return queryset.filter(
                Q(author__user__first_name__icontains=value)
                | Q(author__user__last_name__icontains=value)
                | Q(author__user__username__icontains=value)
            )
        return queryset



from django.db.models import Count, Sum
from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from cart.models import Cart


class CartFilter(filters.FilterSet):
    id = filters.NumberFilter(
        field_name="id",
        lookup_expr="exact",
        help_text=_("Filter by cart ID"),
    )
    user = filters.NumberFilter(
        field_name="user__id",
        lookup_expr="exact",
        help_text=_("Filter by user ID"),
    )
    user_email = filters.CharFilter(
        field_name="user__email",
        lookup_expr="icontains",
        help_text=_("Filter by user email (partial match)"),
    )
    session_key = filters.CharFilter(
        field_name="session_key",
        lookup_expr="exact",
        help_text=_("Filter by session key"),
    )

    is_guest = filters.BooleanFilter(
        method="filter_is_guest",
        help_text=_("Filter guest carts (True) or user carts (False)"),
    )
    has_items = filters.BooleanFilter(
        method="filter_has_items",
        help_text=_("Filter carts that have/don't have items"),
    )
    is_active = filters.BooleanFilter(
        method="filter_is_active",
        help_text=_("Filter active/abandoned carts"),
    )

    min_items = filters.NumberFilter(
        method="filter_min_items",
        help_text=_("Filter carts with at least X items"),
    )
    max_items = filters.NumberFilter(
        method="filter_max_items",
        help_text=_("Filter carts with at most X items"),
    )

    last_activity_after = filters.DateTimeFilter(
        field_name="last_activity",
        lookup_expr="gte",
        help_text=_("Filter carts with last activity after this date"),
    )
    last_activity_before = filters.DateTimeFilter(
        field_name="last_activity",
        lookup_expr="lte",
        help_text=_("Filter carts with last activity before this date"),
    )

    class Meta:
        model = Cart
        fields = [
            "id",
            "user",
            "user_email",
            "session_key",
            "is_guest",
            "has_items",
            "is_active",
            "min_items",
            "max_items",
            "last_activity_after",
            "last_activity_before",
        ]

    def filter_is_guest(self, queryset, name, value):
        if value is True:
            return queryset.filter(user__isnull=True)
        elif value is False:
            return queryset.filter(user__isnull=False)
        return queryset

    def filter_has_items(self, queryset, name, value):
        if value is True:
            return queryset.annotate(item_count=Count("items")).filter(
                item_count__gt=0
            )
        elif value is False:
            return queryset.annotate(item_count=Count("items")).filter(
                item_count=0
            )
        return queryset

    def filter_is_active(self, queryset, name, value):
        if value is True:
            return queryset.active()
        elif value is False:
            return queryset.abandoned()
        return queryset

    def filter_min_items(self, queryset, name, value):
        if value is not None:
            return queryset.annotate(
                total_quantity=Sum("items__quantity")
            ).filter(total_quantity__gte=value)
        return queryset

    def filter_max_items(self, queryset, name, value):
        if value is not None:
            return queryset.annotate(
                total_quantity=Sum("items__quantity")
            ).filter(total_quantity__lte=value)
        return queryset



import django_filters
from django.db.models import Count, Q
from django.utils.translation import gettext_lazy as _

from blog.models.comment import BlogComment


class BlogCommentFilter(django_filters.FilterSet):
    approved = django_filters.BooleanFilter(
        help_text=_("Filter by approval status")
    )
    has_content = django_filters.BooleanFilter(
        method="filter_has_content",
        help_text=_(
            "Filter comments that have content (true) or are empty (false)"
        ),
    )
    content = django_filters.CharFilter(
        field_name="translations__content",
        lookup_expr="icontains",
        help_text=_("Filter by comment content (case-insensitive)"),
    )

    post = django_filters.NumberFilter(
        field_name="post__id", help_text=_("Filter by blog post ID")
    )
    user = django_filters.NumberFilter(
        field_name="user__id", help_text=_("Filter by user ID")
    )
    post__slug = django_filters.CharFilter(
        field_name="post__slug", help_text=_("Filter by blog post slug")
    )

    parent = django_filters.NumberFilter(
        field_name="parent__id", help_text=_("Filter by parent comment ID")
    )
    parent__isnull = django_filters.BooleanFilter(
        field_name="parent",
        lookup_expr="isnull",
        help_text=_("Filter top-level comments (true) or replies (false)"),
    )
    level = django_filters.NumberFilter(
        help_text=_("Filter by comment nesting level (0 for top-level)")
    )
    level__lte = django_filters.NumberFilter(
        field_name="level",
        lookup_expr="lte",
        help_text=_("Filter comments at or above this nesting level"),
    )
    level__gte = django_filters.NumberFilter(
        field_name="level",
        lookup_expr="gte",
        help_text=_("Filter comments at or below this nesting level"),
    )

    has_likes = django_filters.BooleanFilter(
        method="filter_has_likes",
        help_text=_(
            "Filter comments that have likes (true) or no likes (false)"
        ),
    )
    min_likes = django_filters.NumberFilter(
        method="filter_min_likes",
        help_text=_("Filter comments with at least this many likes"),
    )
    has_replies = django_filters.BooleanFilter(
        method="filter_has_replies",
        help_text=_(
            "Filter comments that have replies (true) or no replies (false)"
        ),
    )
    min_replies = django_filters.NumberFilter(
        method="filter_min_replies",
        help_text=_("Filter comments with at least this many replies"),
    )

    user__is_staff = django_filters.BooleanFilter(
        field_name="user__is_staff",
        help_text=_("Filter comments by staff users"),
    )
    user__is_active = django_filters.BooleanFilter(
        field_name="user__is_active",
        help_text=_("Filter comments by active users"),
    )

    post__category = django_filters.NumberFilter(
        field_name="post__category__id",
        help_text=_("Filter by blog post category ID"),
    )
    post__category__slug = django_filters.CharFilter(
        field_name="post__category__slug",
        help_text=_("Filter by blog post category slug"),
    )

    class Meta:
        model = BlogComment
        fields = {
            "id": ["exact", "in"],
            "tree_id": ["exact"],
            "lft": ["gte", "lte"],
            "rght": ["gte", "lte"],
        }

    def filter_has_content(self, queryset, name, value):
        if value is True:
            return queryset.exclude(
                Q(translations__content__isnull=True)
                | Q(translations__content__exact="")
            )
        elif value is False:
            return queryset.filter(
                Q(translations__content__isnull=True)
                | Q(translations__content__exact="")
            )
        return queryset

    def filter_has_likes(self, queryset, name, value):
        if value is True:
            return queryset.annotate(like_count=Count("likes")).filter(
                like_count__gt=0
            )
        elif value is False:
            return queryset.annotate(like_count=Count("likes")).filter(
                like_count=0
            )
        return queryset

    def filter_min_likes(self, queryset, name, value):
        return queryset.annotate(like_count=Count("likes")).filter(
            like_count__gte=value
        )

    def filter_has_replies(self, queryset, name, value):
        if value is True:
            return queryset.annotate(reply_count=Count("children")).filter(
                reply_count__gt=0
            )
        elif value is False:
            return queryset.annotate(reply_count=Count("children")).filter(
                reply_count=0
            )
        return queryset

    def filter_min_replies(self, queryset, name, value):
        return queryset.annotate(reply_count=Count("children")).filter(
            reply_count__gte=value
        )



from __future__ import annotations

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import extend_schema, extend_schema_view
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.filters import SearchFilter
from rest_framework.response import Response

from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from core.utils.views import cache_methods
from region.filters import RegionFilter
from region.models import Region
from region.serializers import (
    RegionDetailSerializer,
    RegionSerializer,
    RegionWriteSerializer,
)


@extend_schema_view(
    **create_schema_view_config(
        model_class=Region,
        display_config={
            "tag": "Regions",
        },
        serializers={
            "list_serializer": RegionSerializer,
            "detail_serializer": RegionDetailSerializer,
            "write_serializer": RegionWriteSerializer,
        },
        error_serializer=ErrorResponseSerializer,
    ),
    get_regions_by_country_alpha_2=extend_schema(
        operation_id="listRegionsByCountry",
        summary=_("Get regions by country"),
        description=_(
            "Get all regions for a specific country using its alpha-2 code."
        ),
        tags=["Geography"],
        responses={
            200: RegionSerializer(many=True),
            404: ErrorResponseSerializer,
        },
    ),
)
@cache_methods(
    settings.DEFAULT_CACHE_TTL,
    methods=["list", "retrieve", "get_regions_by_country_alpha_2"],
)
class RegionViewSet(MultiSerializerMixin, BaseModelViewSet):
    queryset = Region.objects.all()
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_class = RegionFilter
    ordering_fields = ["created_at", "alpha", "sort_order"]
    ordering = ["-created_at"]
    search_fields = ["alpha", "translations__name", "country__alpha_2"]

    serializers = {
        "list": RegionSerializer,
        "create": RegionWriteSerializer,
        "update": RegionWriteSerializer,
        "partial_update": RegionWriteSerializer,
        "retrieve": RegionDetailSerializer,
        "get_regions_by_country_alpha_2": RegionSerializer,
    }

    @action(
        detail=True,
        methods=["GET"],
    )
    def get_regions_by_country_alpha_2(self, request, pk=None, *args, **kwargs):
        regions = Region.objects.filter(country__alpha_2=pk)
        serializer = self.get_serializer(regions, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)



from rest_framework import serializers

from core.api.serializers import ContentObjectRelatedField
from tag.models import TaggedItem
from tag.serializers.tag import TagSerializer


class TaggedItemSerializer(serializers.ModelSerializer[TaggedItem]):
    tag = TagSerializer(read_only=True)
    content_object = ContentObjectRelatedField(read_only=True)

    class Meta:
        model = TaggedItem
        fields = [
            "id",
            "tag",
            "content_type",
            "object_id",
            "content_object",
            "created_at",
            "updated_at",
            "uuid",
        ]
        read_only_fields = (
            "created_at",
            "updated_at",
            "uuid",
        )



from drf_spectacular.helpers import lazy_serializer
from drf_spectacular.utils import extend_schema_field
from parler_rest.serializers import TranslatableModelSerializer
from rest_framework import serializers

from core.api.schema import generate_schema_multi_lang
from core.utils.serializers import TranslatedFieldExtended
from product.models.category import ProductCategory


@extend_schema_field(generate_schema_multi_lang(ProductCategory))
class TranslatedFieldsFieldExtend(TranslatedFieldExtended):
    pass


class ProductCategorySerializer(
    TranslatableModelSerializer, serializers.ModelSerializer[ProductCategory]
):
    translations = TranslatedFieldsFieldExtend(shared_model=ProductCategory)

    class Meta:
        model = ProductCategory
        fields = (
            "id",
            "translations",
            "slug",
            "active",
            "parent",
            "level",
            "tree_id",
            "created_at",
            "updated_at",
            "uuid",
            "recursive_product_count",
        )
        read_only_fields = (
            "id",
            "level",
            "tree_id",
            "created_at",
            "updated_at",
            "uuid",
            "recursive_product_count",
        )


class ProductCategoryDetailSerializer(ProductCategorySerializer):
    children = serializers.SerializerMethodField()

    @extend_schema_field(
        lazy_serializer(
            "product.serializers.category.ProductCategorySerializer"
        )(many=True)
    )
    def get_children(self, obj: ProductCategory):
        if obj.get_children().exists():
            return ProductCategorySerializer(
                obj.get_children(), many=True, context=self.context
            ).data
        return []

    class Meta(ProductCategorySerializer.Meta):
        fields = (
            *ProductCategorySerializer.Meta.fields,
            "children",
            "seo_title",
            "seo_description",
            "seo_keywords",
        )


class ProductCategoryWriteSerializer(
    TranslatableModelSerializer, serializers.ModelSerializer[ProductCategory]
):
    translations = TranslatedFieldsFieldExtend(shared_model=ProductCategory)

    class Meta:
        model = ProductCategory
        fields = (
            "translations",
            "slug",
            "active",
            "parent",
            "seo_title",
            "seo_description",
            "seo_keywords",
        )



from __future__ import annotations

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import (
    extend_schema,
    extend_schema_view,
)
from rest_framework.decorators import action
from rest_framework.filters import SearchFilter
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from core.utils.views import cache_methods
from user.filters.address import UserAddressFilter
from user.models.address import UserAddress
from user.serializers.address import (
    BulkDeleteAddressesRequestSerializer,
    BulkDeleteAddressesResponseSerializer,
    UserAddressDetailSerializer,
    UserAddressSerializer,
    UserAddressWriteSerializer,
    ValidateAddressResponseSerializer,
)


@extend_schema_view(
    **create_schema_view_config(
        model_class=UserAddress,
        display_config={
            "tag": "User Addresses",
        },
        serializers={
            "list_serializer": UserAddressSerializer,
            "detail_serializer": UserAddressDetailSerializer,
            "write_serializer": UserAddressWriteSerializer,
        },
        error_serializer=ErrorResponseSerializer,
    ),
    set_main=extend_schema(
        operation_id="setMainUserAddress",
        summary=_("Set address as main"),
        description=_("Set this address as the user's main address."),
        tags=["User Addresses"],
        request=None,
        responses={
            200: UserAddressDetailSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    get_main=extend_schema(
        operation_id="getMainUserAddress",
        summary=_("Get main address"),
        description=_("Retrieve the user's main address."),
        tags=["User Addresses"],
        responses={
            200: UserAddressDetailSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    validate_address=extend_schema(
        operation_id="validateUserAddress",
        summary=_("Validate address"),
        description=_("Validate an address without saving it."),
        tags=["User Addresses"],
        request=UserAddressWriteSerializer,
        responses={
            200: ValidateAddressResponseSerializer,
            400: ErrorResponseSerializer,
            401: ErrorResponseSerializer,
        },
    ),
    bulk_delete=extend_schema(
        operation_id="bulkDeleteUserAddresses",
        summary=_("Bulk delete addresses"),
        description=_("Delete multiple addresses by their IDs."),
        tags=["User Addresses"],
        request=BulkDeleteAddressesRequestSerializer,
        responses={
            200: BulkDeleteAddressesResponseSerializer,
            400: ErrorResponseSerializer,
            401: ErrorResponseSerializer,
        },
    ),
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class UserAddressViewSet(MultiSerializerMixin, BaseModelViewSet):
    serializer_class = None
    serializers = {
        "default": UserAddressDetailSerializer,
        "list": UserAddressSerializer,
        "retrieve": UserAddressDetailSerializer,
        "create": UserAddressWriteSerializer,
        "update": UserAddressWriteSerializer,
        "partial_update": UserAddressWriteSerializer,
        "set_main": UserAddressDetailSerializer,
        "get_main": UserAddressDetailSerializer,
        "validate_address": ValidateAddressResponseSerializer,
        "bulk_delete": BulkDeleteAddressesResponseSerializer,
    }
    permission_classes = [IsAuthenticated]
    filterset_class = UserAddressFilter
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    ordering_fields = [
        "id",
        "created_at",
        "updated_at",
        "city",
        "zipcode",
        "is_main",
    ]
    ordering = ["-is_main", "-created_at"]
    search_fields = [
        "title",
        "first_name",
        "last_name",
        "street",
        "street_number",
        "city",
        "zipcode",
        "notes",
    ]

    def get_queryset(self):
        if getattr(self, "swagger_fake_view", False):
            return UserAddress.objects.none()

        return UserAddress.objects.filter(
            user=self.request.user
        ).select_related("country", "region")

    @action(detail=True, methods=["POST"])
    def set_main(self, request, pk=None):
        address = self.get_object()
        UserAddress.objects.filter(user=request.user, is_main=True).update(
            is_main=False
        )
        address.is_main = True
        address.save()

        serializer = self.get_serializer(address)
        return Response(serializer.data)

    @action(detail=False, methods=["GET"])
    def get_main(self, request):
        main_address = UserAddress.objects.filter(
            user=request.user, is_main=True
        ).first()
        if main_address:
            serializer = self.get_serializer(main_address)
            return Response(serializer.data)
        return Response({"detail": _("No main address found.")}, status=404)

    @action(detail=False, methods=["POST"])
    def validate_address(self, request):
        serializer = UserAddressWriteSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        return Response(
            {
                "valid": True,
                "details": _("Address validation successful."),
                "data": serializer.validated_data,
            }
        )

    @action(detail=False, methods=["DELETE"])
    def bulk_delete(self, request):
        serializer = BulkDeleteAddressesRequestSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        address_ids = serializer.validated_data["address_ids"]
        deleted_count, _ = UserAddress.objects.filter(
            id__in=address_ids, user=request.user
        ).delete()

        return Response(
            {
                "deleted_count": deleted_count,
                "requested_count": len(address_ids),
                "details": _(
                    f"Deleted {deleted_count} out of {len(address_ids)} requested addresses."
                ),
            }
        )



import logging
from tempfile import NamedTemporaryFile

import requests
from allauth.account.signals import user_signed_up
from django.core.files import File
from django.dispatch import receiver

logger = logging.getLogger(__name__)


@receiver(user_signed_up)
def populate_profile(sociallogin=None, user=None, **kwargs):
    if not sociallogin or not user:
        logger.warning("No sociallogin or user passed to populate_profile")
        return

    provider = sociallogin.account.provider
    picture_url = None

    match provider:
        case "facebook":
            picture_url = f"http://graph.facebook.com/{sociallogin.account.uid}/picture?type=large"
        case "google":
            picture_url = sociallogin.account.extra_data.get("picture")
        case "discord":
            avatar_id = sociallogin.account.extra_data.get("id")
            avatar_hash = sociallogin.account.extra_data.get("avatar")
            if avatar_id and avatar_hash:
                picture_url = f"https://cdn.discordapp.com/avatars/{avatar_id}/{avatar_hash}.png"
            else:
                logger.warning(
                    "Missing avatar_id or avatar_hash for Discord provider"
                )
        case "github":
            picture_url = sociallogin.account.extra_data.get("avatar_url")
        case _:
            logger.warning(f"Unsupported provider: {provider}")

    if picture_url:
        try:
            response = requests.get(picture_url, timeout=10)
            response.raise_for_status()
        except requests.RequestException as e:
            logger.error(f"Failed to retrieve image from {picture_url}: {e}")
            return

        with NamedTemporaryFile(delete=True) as img_temp:
            img_temp.write(response.content)
            img_temp.flush()
            image_filename = (
                f"image_{user.first_name}_{user.last_name}_{user.pk}.jpg"
            )
            user.image.save(image_filename, File(img_temp))



from django.contrib.auth import get_user_model
from django.urls import reverse
from rest_framework import status
from rest_framework.test import APITestCase

from blog.factories.author import BlogAuthorFactory
from blog.factories.category import BlogCategoryFactory
from blog.factories.comment import BlogCommentFactory
from blog.factories.post import BlogPostFactory
from blog.factories.tag import BlogTagFactory
from user.factories.account import UserAccountFactory

User = get_user_model()


class BlogPostFilterTestCase(APITestCase):
    def setUp(self):
        self.user = UserAccountFactory(num_addresses=0)
        self.author = BlogAuthorFactory(user=self.user)

        self.category1 = BlogCategoryFactory(slug="category-1")
        self.category2 = BlogCategoryFactory(slug="category-2")

        self.tag1 = BlogTagFactory(name="Django")
        self.tag2 = BlogTagFactory(name="Python")
        self.tag3 = BlogTagFactory(name="REST")

        self.popular_post = BlogPostFactory(
            author=self.author,
            category=self.category1,
            is_published=True,
            featured=True,
            view_count=5000,
        )
        self.popular_post.tags.set([self.tag1, self.tag2, self.tag3])

        for _i in range(5):
            user = UserAccountFactory(num_addresses=0)
            self.popular_post.likes.add(user)

        for _i in range(20):
            BlogCommentFactory(post=self.popular_post, approved=True)

        self.average_post = BlogPostFactory(
            author=self.author,
            category=self.category1,
            is_published=True,
            featured=False,
            view_count=500,
        )
        self.average_post.tags.set([self.tag1, self.tag2])

        for _i in range(3):
            user = UserAccountFactory(num_addresses=0)
            self.average_post.likes.add(user)

        for _i in range(5):
            BlogCommentFactory(post=self.average_post)

        self.unpopular_post = BlogPostFactory(
            author=self.author,
            category=self.category2,
            is_published=True,
            featured=False,
            view_count=100,
        )
        self.unpopular_post.tags.set([self.tag1])

        user = UserAccountFactory(num_addresses=0)
        self.unpopular_post.likes.add(user)

        BlogCommentFactory(post=self.unpopular_post)

        self.popular_post.refresh_from_db()
        self.average_post.refresh_from_db()
        self.unpopular_post.refresh_from_db()

    def get_post_list_url(self):
        return reverse("blog-post-list")

    def test_filter_by_min_likes(self):
        url = f"{self.get_post_list_url()}?min_likes=4"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(
            response.data["results"][0]["id"], self.popular_post.id
        )

    def test_filter_by_min_comments(self):
        url = f"{self.get_post_list_url()}?min_comments=1"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertGreater(len(response.data["results"]), 0)

    def test_filter_by_min_tags(self):
        url = f"{self.get_post_list_url()}?min_tags=1"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertGreater(len(response.data["results"]), 0)

    def test_filter_by_featured(self):
        url = f"{self.get_post_list_url()}?featured=true"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(
            response.data["results"][0]["id"], self.popular_post.id
        )

    def test_filter_by_min_view_count(self):
        url = f"{self.get_post_list_url()}?min_view_count=800"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(
            response.data["results"][0]["id"], self.popular_post.id
        )

    def test_filter_by_category(self):
        url = f"{self.get_post_list_url()}?category={self.category2.id}"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(
            response.data["results"][0]["id"], self.unpopular_post.id
        )

    def test_filter_by_author(self):
        url = f"{self.get_post_list_url()}?author={self.author.id}"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(len(response.data["results"]), 3)

    def test_filter_by_author_email(self):
        url = f"{self.get_post_list_url()}?author_email={self.user.email}"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(len(response.data["results"]), 3)

    def test_ordering_by_likes_count(self):
        url = f"{self.get_post_list_url()}?ordering=-likesCountField"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        results = response.data["results"]
        self.assertEqual(results[0]["id"], self.popular_post.id)
        self.assertEqual(results[1]["id"], self.average_post.id)
        self.assertEqual(results[2]["id"], self.unpopular_post.id)

    def test_ordering_by_comments_count(self):
        url = f"{self.get_post_list_url()}?ordering=-commentsCountField"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        results = response.data["results"]
        self.assertEqual(results[0]["id"], self.popular_post.id)
        self.assertEqual(results[1]["id"], self.average_post.id)
        self.assertEqual(results[2]["id"], self.unpopular_post.id)

    def test_ordering_by_view_count(self):
        url = f"{self.get_post_list_url()}?ordering=-viewCount"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        results = response.data["results"]
        self.assertEqual(results[0]["id"], self.popular_post.id)
        self.assertEqual(results[1]["id"], self.average_post.id)
        self.assertEqual(results[2]["id"], self.unpopular_post.id)

    def test_combined_filters(self):
        url = f"{self.get_post_list_url()}?min_likes=2&featured=false&min_view_count=200"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(
            response.data["results"][0]["id"], self.average_post.id
        )



from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from user.models.account import UserAccount


class UserAccountFilter(filters.FilterSet):
    id = filters.NumberFilter(
        field_name="id",
        lookup_expr="exact",
        help_text=_("Filter by user account ID"),
    )
    email = filters.CharFilter(
        field_name="email",
        lookup_expr="icontains",
        help_text=_("Filter by email (partial match)"),
    )
    email_exact = filters.CharFilter(
        field_name="email",
        lookup_expr="exact",
        help_text=_("Filter by exact email"),
    )
    username = filters.CharFilter(
        field_name="username",
        lookup_expr="icontains",
        help_text=_("Filter by username (partial match)"),
    )
    username_exact = filters.CharFilter(
        field_name="username",
        lookup_expr="exact",
        help_text=_("Filter by exact username"),
    )
    first_name = filters.CharFilter(
        field_name="first_name",
        lookup_expr="icontains",
        help_text=_("Filter by first name (partial match)"),
    )
    last_name = filters.CharFilter(
        field_name="last_name",
        lookup_expr="icontains",
        help_text=_("Filter by last name (partial match)"),
    )
    is_active = filters.BooleanFilter(
        field_name="is_active",
        help_text=_("Filter by active status"),
    )
    is_staff = filters.BooleanFilter(
        field_name="is_staff",
        help_text=_("Filter by staff status"),
    )
    country = filters.CharFilter(
        field_name="country__alpha_2",
        lookup_expr="exact",
        help_text=_("Filter by country alpha-2 code"),
    )
    country_name = filters.CharFilter(
        field_name="country__translations__name",
        lookup_expr="icontains",
        help_text=_("Filter by country name (partial match)"),
    )
    region = filters.CharFilter(
        field_name="region__alpha",
        lookup_expr="exact",
        help_text=_("Filter by region alpha code"),
    )
    city = filters.CharFilter(
        field_name="city",
        lookup_expr="icontains",
        help_text=_("Filter by city (partial match)"),
    )
    zipcode = filters.CharFilter(
        field_name="zipcode",
        lookup_expr="icontains",
        help_text=_("Filter by zipcode (partial match)"),
    )
    created_after = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="gte",
        help_text=_("Filter users created after this date"),
    )
    created_before = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="lte",
        help_text=_("Filter users created before this date"),
    )
    updated_after = filters.DateTimeFilter(
        field_name="updated_at",
        lookup_expr="gte",
        help_text=_("Filter users updated after this date"),
    )
    updated_before = filters.DateTimeFilter(
        field_name="updated_at",
        lookup_expr="lte",
        help_text=_("Filter users updated before this date"),
    )

    class Meta:
        model = UserAccount
        fields = {
            "id": ["exact"],
            "email": ["exact", "icontains"],
            "username": ["exact", "icontains"],
            "first_name": ["icontains"],
            "last_name": ["icontains"],
            "is_active": ["exact"],
            "is_staff": ["exact"],
            "country": ["exact"],
            "region": ["exact"],
            "city": ["icontains"],
            "zipcode": ["icontains"],
            "created_at": ["gte", "lte"],
            "updated_at": ["gte", "lte"],
        }



import factory
from django.apps import apps
from django.conf import settings
from django.core.files.base import ContentFile
from factory import fuzzy
from factory.django import DjangoModelFactory

from product.enum.category import CategoryImageTypeEnum
from product.factories.category import ProductCategoryFactory
from product.models.category_image import ProductCategoryImage

available_languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]


class ProductCategoryImageTranslationFactory(factory.django.DjangoModelFactory):
    language_code = factory.Iterator(available_languages)
    title = factory.Faker("word")
    alt_text = factory.Faker("sentence")
    master = factory.SubFactory(
        "product.factories.category_image.ProductCategoryImageFactory"
    )

    class Meta:
        model = apps.get_model("product", "ProductCategoryImageTranslation")
        django_get_or_create = ("language_code", "master")


class ProductCategoryImageFactory(DjangoModelFactory):
    class Meta:
        model = ProductCategoryImage
        skip_postgeneration_save = True

    category = factory.SubFactory(ProductCategoryFactory)
    image_type = fuzzy.FuzzyChoice(
        CategoryImageTypeEnum.choices, getter=lambda c: c[0]
    )
    active = True
    sort_order = factory.Sequence(lambda n: n)

    image = factory.LazyAttribute(
        lambda obj: ContentFile(
            factory.django.ImageField()._make_data(
                {"width": 300, "height": 200, "color": "blue"}
            ),
            name=f"category_{obj.category.id}_{obj.image_type}.jpg",
        )
    )

    class Params:
        main_image = factory.Trait(
            image_type=CategoryImageTypeEnum.MAIN,
            sort_order=1,
        )
        banner_image = factory.Trait(
            image_type=CategoryImageTypeEnum.BANNER,
            sort_order=2,
        )
        icon_image = factory.Trait(
            image_type=CategoryImageTypeEnum.ICON,
            sort_order=3,
        )
        inactive = factory.Trait(active=False)

    @factory.post_generation
    def translations(self, create, extracted, **kwargs):
        if not create:
            return

        translations = extracted or [
            ProductCategoryImageTranslationFactory(
                language_code=lang, master=self
            )
            for lang in available_languages
        ]

        for translation in translations:
            translation.master = self
            translation.save()



import re

from rest_framework.filters import OrderingFilter


class PascalSnakeCaseOrderingFilter(OrderingFilter):
    def get_ordering(self, request, queryset, view):
        if "query_params" not in dir(request):
            return self.get_default_ordering(view)

        ordering = request.query_params.get(self.ordering_param)
        if ordering:
            ordering = (
                re.sub("([a-z])([A-Z])", r"\1_\2", ordering).lower().lstrip("_")
            )
            return [term for term in ordering.split(",")]
        return self.get_default_ordering(view)



import datetime
import os
from os import makedirs, path, remove
from unittest.mock import patch

from django.conf import settings
from django.test import TestCase

from core import celery
from core.tasks import clear_log_files_task


class CeleryConfigTestCase(TestCase):
    @patch("core.celery.Celery")
    def test_create_celery_app(self, mock_celery):
        mock_celery_instance = mock_celery.return_value

        app = celery.create_celery_app()

        self.assertTrue(mock_celery.called)
        self.assertEqual(app.conf.enable_utc, False)
        self.assertEqual(app.conf.timezone, mock_celery_instance.conf.timezone)
        self.assertTrue(app.autodiscover_tasks.called)

    def tearDown(self):
        celery.app = None
        celery.debug_task = None
        super().tearDown()


class CleanupLogFilesTaskTest(TestCase):
    days = 30

    def setUp(self):
        self.logs_path = path.join(settings.BASE_DIR, "logs")
        makedirs(self.logs_path, exist_ok=True)

        self.old_file = path.join(self.logs_path, "logs_01-01-2000.log")
        self.new_file = path.join(self.logs_path, "logs_01-01-3000.log")

        with open(self.old_file, "w") as f:
            f.write("Old file")

        with open(self.new_file, "w") as f:
            f.write("New file")

        old_time = (
            datetime.datetime.now(tz=datetime.UTC) - datetime.timedelta(days=31)
        ).timestamp()
        os.utime(self.old_file, (old_time, old_time))

    def tearDown(self):
        if path.exists(self.old_file):
            remove(self.old_file)
        if path.exists(self.new_file):
            remove(self.new_file)
        super().tearDown()

    def test_cleanup_log_files_task(self):
        clear_log_files_task(self.days)

        self.assertFalse(path.exists(self.old_file))
        self.assertTrue(path.exists(self.new_file))

    def test_does_not_remove_recent_logs(self):
        recent_file = path.join(self.logs_path, "logs_01-01-2021.log")
        with open(recent_file, "w") as f:
            f.write("Recent file")

        recent_time = (
            datetime.datetime.now(tz=datetime.UTC)
            - datetime.timedelta(days=self.days - 1)
        ).timestamp()
        os.utime(recent_file, (recent_time, recent_time))

        clear_log_files_task(self.days)

        self.assertTrue(path.exists(recent_file))

        if path.exists(recent_file):
            remove(recent_file)



import logging
from collections.abc import Callable

import factory
from django.conf import settings
from django.core.exceptions import ValidationError
from django.db.models import Model
from faker import Faker

logger = logging.getLogger(__name__)

fake = Faker()


class TranslationUtilities:
    """Utilities for handling django-parler translations in factories"""

    @staticmethod
    def get_available_languages():
        """Get available languages from django-parler settings"""
        try:
            return [
                lang["code"]
                for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
            ]
        except (AttributeError, KeyError):
            logger.warning(
                "PARLER_LANGUAGES not properly configured, using default language"
            )
            return [settings.LANGUAGE_CODE]

    @staticmethod
    def is_translation_factory(factory_class):
        """Check if a factory is for a translation model"""
        if not hasattr(factory_class, "_meta") or not hasattr(
            factory_class._meta, "model"
        ):
            return False

        model_name = factory_class._meta.model.__name__
        return model_name.endswith("Translation")

    @staticmethod
    def get_master_factory_from_translation(factory_class):
        """Get the master factory from a translation factory"""
        if not TranslationUtilities.is_translation_factory(factory_class):
            return None

        # Look for master field in factory
        if hasattr(factory_class, "master"):
            master_field = factory_class.master
            if hasattr(master_field, "_factory"):
                return master_field._factory

        return None

    @staticmethod
    def has_translations(factory_class):
        """Check if a factory creates translations via post_generation"""
        if not hasattr(factory_class, "translations"):
            return False

        # Check if it's a post_generation method
        translations_attr = factory_class.translations
        return (
            hasattr(translations_attr, "_decorator")
            and translations_attr._decorator == "post_generation"
        )

    @staticmethod
    def validate_translation_completeness(instance, model_class):
        """Validate that all required translations exist for an instance"""
        if not hasattr(model_class, "_parler_meta"):
            return True  # Not a translatable model

        available_languages = TranslationUtilities.get_available_languages()
        translation_model = model_class._parler_meta.root_model

        existing_languages = set(
            translation_model.objects.filter(master=instance).values_list(
                "language_code", flat=True
            )
        )

        missing_languages = set(available_languages) - existing_languages

        if missing_languages:
            logger.warning(
                f"Missing translations for {model_class.__name__} instance {instance.pk}: {missing_languages}"
            )
            return False

        return True


class UniqueFieldMixin:
    @classmethod
    def generate_unique_field(
        cls,
        model: type[Model],
        field_name: str,
        generator_func: Callable[[], any],
        max_attempts=20,
    ):
        attempts = 0
        while attempts < max_attempts:
            value = generator_func()
            if not model.objects.filter(**{field_name: value}).exists():
                return value
            attempts += 1
        logger.error(
            f"Failed to generate unique '{field_name}' for {model.__name__} after {max_attempts} attempts."
        )
        raise ValidationError(
            f"Unable to generate unique value for {field_name} on {model.__name__} after {max_attempts} attempts."
        )

    @classmethod
    def get_unique_value(
        cls,
        model: type[Model],
        field_name: str,
        generator_func: Callable[[], any],
    ):
        return cls.generate_unique_field(model, field_name, generator_func)


class CustomDjangoModelFactory(factory.django.DjangoModelFactory):
    """Enhanced base factory with translation support, and dependency management."""

    depends_on = []  # Explicit factory dependencies
    conditional_depends_on = {}  # Conditional dependencies: {'condition': ['FactoryName']}
    execution_priority = 0  # Higher number = executed later

    locale_aware = True  # Enable locale-specific data
    relationship_aware = False  # Enable relationship intelligence

    class Meta:
        abstract = True
        exclude = (
            "depends_on",
            "conditional_depends_on",
            "execution_priority",
            "locale_aware",
            "relationship_aware",
            "unique_model_fields",
        )

    @classmethod
    def _create(cls, model_class: type[Model], *args, **kwargs):
        # Handle unique fields
        if hasattr(cls, "unique_model_fields"):
            for field, generator_func in cls.unique_model_fields:
                if field not in kwargs:
                    kwargs[field] = UniqueFieldMixin.get_unique_value(
                        model_class, field, generator_func
                    )

        # Create the instance first
        instance = super()._create(model_class, *args, **kwargs)

        # This is now handled by the command, not here

        return instance

    @classmethod
    def create_with_translation_validation(cls, **kwargs):
        """Create an instance and validate translation completeness"""
        instance = cls.create(**kwargs)

        # Validate translations if this is a translatable model
        if TranslationUtilities.validate_translation_completeness(
            instance, cls._meta.model
        ):
            logger.debug(
                f"Successfully created {cls._meta.model.__name__} with complete translations"
            )

        return instance



from __future__ import annotations

import contextlib
import logging
import uuid

from django.conf import settings
from django.db import transaction
from django.utils.translation import gettext_lazy as _
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import extend_schema, extend_schema_view
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.exceptions import (
    NotAuthenticated,
    NotFound,
    PermissionDenied,
    ValidationError,
)
from rest_framework.filters import SearchFilter
from rest_framework.permissions import IsAdminUser, IsAuthenticated
from rest_framework.response import Response

from core.api.permissions import IsSelfOrAdmin
from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from core.utils.views import cache_methods
from order.enum.status import OrderStatus
from order.filters import OrderFilter
from order.models.order import Order
from order.serializers.order import (
    OrderDetailSerializer,
    OrderSerializer,
    OrderWriteSerializer,
)
from order.services import OrderService, OrderServiceError


@extend_schema_view(
    **create_schema_view_config(
        model_class=Order,
        display_config={
            "tag": "Orders",
        },
        serializers={
            "list_serializer": OrderSerializer,
            "detail_serializer": OrderDetailSerializer,
            "write_serializer": OrderWriteSerializer,
        },
    ),
    retrieve_by_uuid=extend_schema(
        operation_id="retrieveOrderByUuid",
        summary=_("Retrieve an order by UUID"),
        description=_(
            "Get detailed information about a specific order using its UUID"
        ),
        tags=["Orders"],
        responses={
            200: OrderDetailSerializer,
            400: ErrorResponseSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    cancel=extend_schema(
        operation_id="cancelOrder",
        summary=_("Cancel an order"),
        description=_("Cancel an existing order and restore product stock"),
        tags=["Orders"],
        responses={
            200: OrderDetailSerializer,
            400: ErrorResponseSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    my_orders=extend_schema(
        operation_id="listMyOrders",
        summary=_("List current user's orders"),
        description=_("Returns a list of the authenticated user's orders"),
        tags=["Orders"],
        responses={
            200: OrderSerializer(many=True),
            400: ErrorResponseSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    add_tracking=extend_schema(
        operation_id="addOrderTracking",
        summary=_("Add tracking information to an order"),
        description=_("Add tracking information to an existing order"),
        tags=["Orders"],
        responses={
            200: OrderDetailSerializer,
            400: ErrorResponseSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
    update_status=extend_schema(
        operation_id="updateOrderStatus",
        summary=_("Update the status of an order"),
        description=_("Update the status of an existing order"),
        tags=["Orders"],
        responses={
            200: OrderDetailSerializer,
            400: ErrorResponseSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    ),
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class OrderViewSet(MultiSerializerMixin, BaseModelViewSet):
    queryset = Order.objects.all()
    serializers = {
        "default": OrderDetailSerializer,
        "list": OrderSerializer,
        "retrieve": OrderDetailSerializer,
        "create": OrderWriteSerializer,
        "update": OrderWriteSerializer,
        "partial_update": OrderWriteSerializer,
        "retrieve_by_uuid": OrderDetailSerializer,
        "cancel": OrderDetailSerializer,
        "add_tracking": OrderDetailSerializer,
        "update_status": OrderDetailSerializer,
        "my_orders": OrderSerializer,
    }
    response_serializers = {
        "create": OrderDetailSerializer,
        "update": OrderDetailSerializer,
        "partial_update": OrderDetailSerializer,
    }
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_class = OrderFilter
    ordering_fields = [
        "id",
        "created_at",
        "updated_at",
        "status",
        "status_updated_at",
        "paid_amount",
        "shipping_price",
        "payment_status",
        "user__first_name",
        "user__last_name",
    ]
    ordering = ["-created_at"]
    search_fields = [
        "user__email",
        "user__username",
        "user__first_name",
        "user__last_name",
        "first_name",
        "last_name",
        "email",
        "phone",
        "mobile_phone",
        "city",
        "street",
        "zipcode",
        "tracking_number",
        "payment_id",
    ]
    permission_classes = [IsAuthenticated, IsSelfOrAdmin]

    def get_permissions(self):
        if self.action in [
            "list",
            "retrieve",
            "update",
            "partial_update",
            "destroy",
        ]:
            self.permission_classes = [IsSelfOrAdmin]
        elif self.action in ["add_tracking", "update_status"]:
            self.permission_classes = [IsAdminUser]
        elif self.action == "create":
            self.permission_classes = []
        else:
            self.permission_classes = [IsAuthenticated]

        return super().get_permissions()

    def check_object_permissions(self, request, obj):
        super().check_object_permissions(request, obj)

        if request.user.is_staff:
            return

        if obj.user_id != request.user.id:
            raise PermissionDenied(
                _("You do not have permission to access this order.")
            )

    def get_object(self):
        lookup_url_kwarg = self.lookup_url_kwarg or self.lookup_field
        order_id = self.kwargs.get(lookup_url_kwarg)
        max_order_id_length = 8

        try:
            if (
                isinstance(order_id, str)
                and len(order_id) > max_order_id_length
                and "-" in order_id
            ):
                try:
                    uuid.UUID(order_id)
                    return OrderService.get_order_by_uuid(order_id)
                except (ValueError, TypeError):
                    pass

            return OrderService.get_order_by_id(int(order_id))
        except Order.DoesNotExist as e:
            raise NotFound(
                _("Order with ID {order_id} not found").format(
                    order_id=order_id
                )
            ) from e

    @transaction.atomic
    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(
            data=request.data, context={"request": request}
        )

        if not serializer.is_valid():
            return Response(
                serializer.errors, status=status.HTTP_400_BAD_REQUEST
            )

        try:
            user = request.user if request.user.is_authenticated else None
            validated_data = serializer.validated_data.copy()

            if user and "user" not in validated_data:
                validated_data["user"] = user

            order = serializer.create(validated_data)

            result_serializer = OrderDetailSerializer(
                order, context={"request": request}
            )
            return Response(
                result_serializer.data, status=status.HTTP_201_CREATED
            )

        except OrderServiceError as e:
            logger = logging.getLogger(__name__)

            logger.error(
                "Error creating order: %s",
                str(e),
            )
            raise ValidationError(
                {
                    "detail": _(
                        "An error occurred while processing your request."
                    )
                }
            ) from e
        except Exception as e:
            logger = logging.getLogger(__name__)

            logger.error(
                "Error creating order: %s",
                str(e),
            )
            return Response(
                {
                    "detail": _(
                        "An error occurred while processing your request."
                    )
                },
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )

    @action(detail=True, methods=["GET"])
    def retrieve_by_uuid(self, request, *args, **kwargs):
        uuid_str = kwargs.get("uuid")
        if not uuid_str:
            raise NotFound(_("UUID parameter is required"))

        try:
            order = OrderService.get_order_by_uuid(uuid_str)
        except Order.DoesNotExist as e:
            raise NotFound(
                _("Order with UUID {uuid} not found").format(uuid=uuid_str)
            ) from e

        self.check_object_permissions(request, order)
        serializer = self.get_serializer(order)
        return Response(serializer.data)

    @action(detail=True, methods=["POST"])
    def cancel(self, request, *args, **kwargs):
        order = self.get_object()

        user = request.user
        if not user.is_staff and (not order.user or order.user.id != user.id):
            raise PermissionDenied(
                _("You do not have permission to cancel this order")
            )

        try:
            canceled_order = OrderService.cancel_order(order)
            serializer = self.get_serializer(canceled_order)
            return Response(serializer.data)
        except OrderServiceError as e:
            logger = logging.getLogger(__name__)

            logger.error(f"Error canceling order: {e}")
            raise ValidationError(
                {
                    "detail": _(
                        "An error occurred while processing your request."
                    )
                }
            ) from e
        except Exception as e:
            logger = logging.getLogger(__name__)

            logger.error(
                "Error canceling order: %s",
                str(e),
            )

            return Response(
                {"detail": _("An unexpected error occurred")},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )

    @action(detail=False, methods=["GET"])
    def my_orders(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            raise NotAuthenticated(
                _("Authentication is required to view your orders")
            )

        user_orders = OrderService.get_user_orders(request.user.id)

        filtered_qs = self.filter_queryset(user_orders)

        page = self.paginate_queryset(filtered_qs)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(filtered_qs, many=True)
        return Response(serializer.data)

    @action(detail=True, methods=["POST"])
    def add_tracking(self, request, *args, **kwargs):
        order = self.get_object()

        if not request.user.is_staff:
            raise PermissionDenied(_("Only staff can add tracking information"))

        tracking_number = request.data.get("tracking_number")
        shipping_carrier = request.data.get("shipping_carrier")

        if not tracking_number:
            raise ValidationError(
                {"tracking_number": _("Tracking number is required")}
            )

        if not shipping_carrier:
            raise ValidationError(
                {"shipping_carrier": _("Shipping carrier is required")}
            )

        try:
            order.add_tracking_info(tracking_number, shipping_carrier)

            if (
                order.status
                in {
                    OrderStatus.DELIVERED,
                    OrderStatus.COMPLETED,
                    OrderStatus.RETURNED,
                    OrderStatus.REFUNDED,
                }
                or order.status == OrderStatus.SHIPPED
            ):
                pass
            elif order.status == OrderStatus.PROCESSING:
                OrderService.update_order_status(order, OrderStatus.SHIPPED)
            elif order.status == OrderStatus.PENDING:
                OrderService.update_order_status(order, OrderStatus.PROCESSING)
                OrderService.update_order_status(order, OrderStatus.SHIPPED)
            else:
                with contextlib.suppress(OrderServiceError):
                    OrderService.update_order_status(order, OrderStatus.SHIPPED)

            order = OrderService.get_order_by_id(order.id)
            serializer = self.get_serializer(order)
            return Response(serializer.data)

        except OrderServiceError as e:
            logger = logging.getLogger(__name__)

            logger.error(f"Error adding tracking information: {e}")
            raise ValidationError(
                {
                    "detail": _(
                        "An error occurred while processing your request."
                    )
                }
            ) from e
        except Exception as e:
            logger = logging.getLogger(__name__)

            logger.error(
                "Error adding tracking information: %s",
                str(e),
            )
            return Response(
                {"detail": _("An unexpected error occurred")},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )

    @action(detail=True, methods=["POST"])
    def update_status(self, request, *args, **kwargs):
        order = self.get_object()

        if not request.user.is_staff:
            raise PermissionDenied(_("Only staff can update order status"))

        status_value = request.data.get("status")
        if not status_value:
            raise ValidationError({"status": _("Status is required")})

        try:
            OrderService.update_order_status(order, status_value)

            order = OrderService.get_order_by_id(order.id)

            serializer = self.get_serializer(order)
            return Response(serializer.data)
        except ValueError as e:
            logger = logging.getLogger(__name__)

            logger.error(f"Error updating order status: {e}")
            raise ValidationError(
                {
                    "detail": _(
                        "An error occurred while processing your request."
                    )
                }
            ) from e
        except OrderServiceError as e:
            logger = logging.getLogger(__name__)

            logger.error(f"Error updating order status: {e}")
            raise ValidationError(
                {
                    "detail": _(
                        "An error occurred while processing your request."
                    )
                }
            ) from e
        except Exception as e:
            logger = logging.getLogger(__name__)

            logger.error(
                "Error updating order status: %s",
                str(e),
            )
            return Response(
                {"detail": _("An unexpected error occurred")},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )



from django.urls import reverse
from rest_framework.test import APITestCase

from blog.models.post import BlogPost
from blog.views.post import BlogPostViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter


class PascalSnakeCaseOrderingFilterTest(APITestCase):
    def setUp(self):
        self.view = BlogPostViewSet.as_view({"get": "list"})
        self.filter = PascalSnakeCaseOrderingFilter()

    def create_request(self, ordering_value):
        url = reverse("blog-post-list")
        return self.client.get(url, {"ordering": ordering_value})

    def test_get_ordering_pascal_case(self):
        request = self.create_request("CreatedAt")
        queryset = BlogPost.objects.all()
        result = self.filter.get_ordering(
            request.renderer_context["request"], queryset, self.view
        )
        self.assertEqual(result, ["created_at"])

    def test_get_ordering_mixed_case(self):
        request = self.create_request("CreatedAt,UpdatedAt")
        queryset = BlogPost.objects.all()
        result = self.filter.get_ordering(
            request.renderer_context["request"], queryset, self.view
        )
        self.assertEqual(result, ["created_at", "updated_at"])

    def test_get_ordering_snake_case(self):
        request = self.create_request("created_at,updated_at")
        queryset = BlogPost.objects.all()
        result = self.filter.get_ordering(
            request.renderer_context["request"], queryset, self.view
        )
        self.assertEqual(result, ["created_at", "updated_at"])

    def test_get_ordering_empty(self):
        request = self.create_request("")
        queryset = BlogPost.objects.all()
        result = self.filter.get_ordering(
            request.renderer_context["request"], queryset, self.view
        )
        default_ordering = self.filter.get_default_ordering(self.view)
        self.assertEqual(result, default_ordering)



from datetime import timedelta

from django.contrib import admin
from django.db.models.functions import Length
from django.utils import timezone
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from unfold.admin import ModelAdmin
from unfold.contrib.filters.admin import (
    RangeDateTimeFilter,
)

from contact.models import Contact
from core.admin import ExportModelAdmin


class MessageLengthFilter(admin.SimpleListFilter):
    title = _("Message Length")
    parameter_name = "message_length"

    def lookups(self, request, model_admin):
        return [
            ("short", _("Short (<100 chars)")),
            ("medium", _("Medium (100-500 chars)")),
            ("long", _("Long (>500 chars)")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "short":
            return queryset.annotate(msg_len=Length("message")).filter(
                msg_len__lt=100
            )
        elif self.value() == "medium":
            return queryset.annotate(msg_len=Length("message")).filter(
                msg_len__range=(100, 500)
            )
        elif self.value() == "long":
            return queryset.annotate(msg_len=Length("message")).filter(
                msg_len__gt=500
            )
        return queryset


class RecentContactFilter(admin.SimpleListFilter):
    title = _("Contact Period")
    parameter_name = "contact_period"

    def lookups(self, request, model_admin):
        return [
            ("today", _("Today")),
            ("week", _("This Week")),
            ("month", _("This Month")),
            ("quarter", _("This Quarter")),
        ]

    def queryset(self, request, queryset):
        now = timezone.now()

        if self.value() == "today":
            start_date = now.replace(hour=0, minute=0, second=0, microsecond=0)
            return queryset.filter(created_at__gte=start_date)
        elif self.value() == "week":
            start_date = now - timedelta(days=7)
            return queryset.filter(created_at__gte=start_date)
        elif self.value() == "month":
            start_date = now - timedelta(days=30)
            return queryset.filter(created_at__gte=start_date)
        elif self.value() == "quarter":
            start_date = now - timedelta(days=90)
            return queryset.filter(created_at__gte=start_date)
        return queryset


@admin.register(Contact)
class ContactAdmin(ExportModelAdmin, ModelAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = True
    list_filter_submit = True
    list_filter_sheet = True

    list_display = [
        "id",
        "contact_info",
        "message_preview",
        "message_stats",
        "contact_timing",
        "priority_badge",
    ]
    list_filter = [
        RecentContactFilter,
        MessageLengthFilter,
        ("created_at", RangeDateTimeFilter),
        ("updated_at", RangeDateTimeFilter),
    ]
    search_fields = [
        "name",
        "email",
        "message",
    ]
    readonly_fields = (
        "id",
        "uuid",
        "created_at",
        "updated_at",
        "contact_analytics",
        "message_analytics",
        "timing_info",
    )
    list_per_page = 25
    date_hierarchy = "created_at"

    fieldsets = (
        (
            _("Contact Information"),
            {
                "fields": ("name", "email"),
                "classes": ("wide",),
            },
        ),
        (
            _("Message"),
            {
                "fields": ("message",),
                "classes": ("wide",),
            },
        ),
        (
            _("Analytics"),
            {
                "fields": ("contact_analytics", "message_analytics"),
                "classes": ("collapse",),
            },
        ),
        (
            _("Timing Information"),
            {
                "fields": ("timing_info",),
                "classes": ("collapse",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("id", "uuid", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def get_queryset(self, request):
        return super().get_queryset(request)

    def get_ordering(self, request):
        return ["-created_at", "name"]

    def contact_info(self, obj):
        is_suspicious = (
            "@" not in obj.email or "." not in obj.email.split("@")[-1]
        )

        email_class = (
            "text-red-600 dark:text-red-400"
            if is_suspicious
            else "text-base-600 dark:text-base-400"
        )

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="{}">{}</div>'
            '<div class="text-xs text-base-500 dark:text-base-400">ID: {}</div>'
            "</div>",
            obj.name[:30] + ("..." if len(obj.name) > 30 else ""),
            email_class,
            obj.email,
            obj.id,
        )

    contact_info.short_description = _("Contact")

    def message_preview(self, obj):
        message = obj.message
        preview = message[:100] + "..." if len(message) > 100 else message

        preview = preview.replace("\n", " ").replace("\r", " ")

        return format_html(
            '<div class="text-sm">'
            '<div class="text-base-900 dark:text-base-100" title="{}">{}</div>'
            "</div>",
            message.replace('"', "&quot;"),
            preview,
        )

    message_preview.short_description = _("Message")

    def message_stats(self, obj):
        char_count = len(obj.message)
        word_count = len(obj.message.split())
        line_count = obj.message.count("\n") + 1

        if char_count < 100:
            length_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-200 rounded-full">'
                "📝 Short"
                "</span>"
            )
        elif char_count < 500:
            length_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-200 rounded-full">'
                "📄 Medium"
                "</span>"
            )
        else:
            length_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-200 rounded-full">'
                "📋 Long"
                "</span>"
            )

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{} chars</div>'
            '<div class="text-base-600 dark:text-base-400">{} words</div>'
            '<div class="text-base-500 dark:text-base-400">{} lines</div>'
            '<div class="mt-1">{}</div>'
            "</div>",
            char_count,
            word_count,
            line_count,
            length_badge,
        )

    message_stats.short_description = _("Message Stats")

    def contact_timing(self, obj):
        now = timezone.now()
        diff = now - obj.created_at

        if diff < timedelta(hours=1):
            time_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-200 rounded-full">'
                "🚨 Just Now"
                "</span>"
            )
        elif diff < timedelta(days=1):
            hours_ago = int(diff.total_seconds() // 3600)
            time_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-orange-50 dark:bg-orange-900 text-orange-700 dark:text-orange-200 rounded-full">'
                "🕒 {}h ago"
                "</span>",
                hours_ago,
            )
        elif diff < timedelta(days=7):
            time_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-200 rounded-full">'
                "📅 {}d ago"
                "</span>",
                diff.days,
            )
        else:
            time_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-gray-50 dark:bg-gray-800 text-base-700 dark:text-base-200 rounded-full">'
                "📆 Old"
                "</span>"
            )

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="mt-1">{}</div>'
            "</div>",
            obj.created_at.strftime("%Y-%m-%d"),
            time_badge,
        )

    contact_timing.short_description = _("Timing")

    def priority_badge(self, obj):
        now = timezone.now()
        diff = now - obj.created_at
        message_length = len(obj.message)

        if diff < timedelta(hours=2):
            priority = "urgent"
        elif message_length > 500:
            priority = "high"
        elif diff < timedelta(days=1):
            priority = "medium"
        else:
            priority = "low"

        priority_config = {
            "urgent": {
                "bg": "bg-red-50 dark:bg-red-900",
                "text": "text-red-700 dark:text-red-200",
                "icon": "🚨",
                "label": "Urgent",
            },
            "high": {
                "bg": "bg-orange-50 dark:bg-orange-900",
                "text": "text-orange-700 dark:text-orange-200",
                "icon": "⚡",
                "label": "High",
            },
            "medium": {
                "bg": "bg-yellow-50 dark:bg-yellow-900",
                "text": "text-yellow-700 dark:text-yellow-200",
                "icon": "⚠️",
                "label": "Medium",
            },
            "low": {
                "bg": "bg-green-50 dark:bg-green-900",
                "text": "text-green-700 dark:text-green-200",
                "icon": "✅",
                "label": "Low",
            },
        }

        config = priority_config[priority]

        return format_html(
            '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium {} {} rounded-full gap-1">'
            "<span>{}</span>"
            "<span>{}</span>"
            "</span>",
            config["bg"],
            config["text"],
            config["icon"],
            config["label"],
        )

    priority_badge.short_description = _("Priority")

    def contact_analytics(self, obj):
        now = timezone.now()
        age = now - obj.created_at
        word_count = len(obj.message.split())

        reading_time = max(1, word_count // 200)

        return format_html(
            '<div class="text-sm">'
            '<div class="grid grid-cols-2 gap-2">'
            "<div><strong>Contact Age:</strong></div><div>{}d {}h</div>"
            "<div><strong>Response Time:</strong></div><div>{}</div>"
            "<div><strong>Email Domain:</strong></div><div>{}</div>"
            "<div><strong>Contact Day:</strong></div><div>{}</div>"
            "<div><strong>Contact Time:</strong></div><div>{}</div>"
            "<div><strong>Reading Time:</strong></div><div>{}min</div>"
            "</div>"
            "</div>",
            age.days,
            age.seconds // 3600,
            "Pending" if age < timedelta(days=1) else "Delayed",
            obj.email.split("@")[-1] if "@" in obj.email else "Invalid",
            obj.created_at.strftime("%A"),
            obj.created_at.strftime("%H:%M"),
            reading_time,
        )

    contact_analytics.short_description = _("Contact Analytics")

    def message_analytics(self, obj):
        message = obj.message
        char_count = len(message)
        word_count = len(message.split())
        sentence_count = (
            message.count(".") + message.count("!") + message.count("?")
        )
        paragraph_count = message.count("\n\n") + 1

        urgent_words = [
            "urgent",
            "emergency",
            "asap",
            "immediately",
            "help",
            "problem",
            "issue",
            "broken",
        ]
        urgency_score = sum(
            1 for word in urgent_words if word.lower() in message.lower()
        )

        return format_html(
            '<div class="text-sm">'
            '<div class="grid grid-cols-2 gap-2">'
            "<div><strong>Characters:</strong></div><div>{}</div>"
            "<div><strong>Words:</strong></div><div>{}</div>"
            "<div><strong>Sentences:</strong></div><div>{}</div>"
            "<div><strong>Paragraphs:</strong></div><div>{}</div>"
            "<div><strong>Urgency Score:</strong></div><div>{}/10</div>"
            "<div><strong>Avg Word Length:</strong></div><div>{:.1f}</div>"
            "</div>"
            "</div>",
            char_count,
            word_count,
            sentence_count,
            paragraph_count,
            urgency_score,
            char_count / max(word_count, 1),
        )

    message_analytics.short_description = _("Message Analytics")

    def timing_info(self, obj):
        now = timezone.now()
        created_age = now - obj.created_at
        updated_age = now - obj.updated_at

        is_business_hours = 9 <= obj.created_at.hour <= 17
        is_weekend = obj.created_at.weekday() >= 5

        return format_html(
            '<div class="text-sm">'
            '<div class="grid grid-cols-2 gap-2">'
            "<div><strong>Created:</strong></div><div>{}</div>"
            "<div><strong>Updated:</strong></div><div>{}</div>"
            "<div><strong>Age:</strong></div><div>{}d {}h</div>"
            "<div><strong>Last Modified:</strong></div><div>{}d ago</div>"
            "<div><strong>Business Hours:</strong></div><div>{}</div>"
            "<div><strong>Weekend Contact:</strong></div><div>{}</div>"
            "<div><strong>Time Zone:</strong></div><div>UTC</div>"
            "<div><strong>Season:</strong></div><div>{}</div>"
            "</div>"
            "</div>",
            obj.created_at.strftime("%Y-%m-%d %H:%M:%S"),
            obj.updated_at.strftime("%Y-%m-%d %H:%M:%S"),
            created_age.days,
            created_age.seconds // 3600,
            updated_age.days,
            "Yes" if is_business_hours else "No",
            "Yes" if is_weekend else "No",
            self._get_season(obj.created_at.month),
        )

    timing_info.short_description = _("Timing Information")

    def _get_season(self, month):
        if month in [12, 1, 2]:
            return "Winter"
        elif month in [3, 4, 5]:
            return "Spring"
        elif month in [6, 7, 8]:
            return "Summer"
        else:
            return "Autumn"



import django_filters

from notification.models.user import NotificationUser


class NotificationUserFilter(django_filters.FilterSet):
    id = django_filters.NumberFilter(field_name="id")
    user = django_filters.NumberFilter(field_name="user")
    notification = django_filters.NumberFilter(field_name="notification")
    seen = django_filters.BooleanFilter(field_name="seen")

    seen_after = django_filters.DateTimeFilter(
        field_name="seen_at", lookup_expr="gte"
    )
    seen_before = django_filters.DateTimeFilter(
        field_name="seen_at", lookup_expr="lte"
    )
    created_after = django_filters.DateTimeFilter(
        field_name="created_at", lookup_expr="gte"
    )
    created_before = django_filters.DateTimeFilter(
        field_name="created_at", lookup_expr="lte"
    )

    notification_kind = django_filters.CharFilter(
        field_name="notification__kind"
    )
    notification_category = django_filters.CharFilter(
        field_name="notification__category"
    )
    notification_priority = django_filters.CharFilter(
        field_name="notification__priority"
    )

    has_seen_at = django_filters.BooleanFilter(method="filter_has_seen_at")

    class Meta:
        model = NotificationUser
        fields = [
            "id",
            "user",
            "notification",
            "seen",
            "seen_after",
            "seen_before",
            "created_after",
            "created_before",
            "notification_kind",
            "notification_category",
            "notification_priority",
            "has_seen_at",
        ]

    def filter_has_seen_at(self, queryset, name, value):
        if value is True:
            return queryset.filter(seen_at__isnull=False)
        elif value is False:
            return queryset.filter(seen_at__isnull=True)
        return queryset



from __future__ import annotations

from django.utils.translation import gettext_lazy as _
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import extend_schema, extend_schema_view
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.filters import SearchFilter
from rest_framework.response import Response

from core.api.views import BaseModelViewSet
from core.filters.custom_filters import PascalSnakeCaseOrderingFilter
from core.utils.serializers import (
    MultiSerializerMixin,
    create_schema_view_config,
)
from product.models.favourite import ProductFavourite
from product.serializers.favourite import (
    ProductDetailResponseSerializer,
    ProductFavouriteByProductsRequestSerializer,
    ProductFavouriteByProductsResponseSerializer,
    ProductFavouriteDetailSerializer,
    ProductFavouriteSerializer,
    ProductFavouriteWriteSerializer,
)

schema_config = create_schema_view_config(
    model_class=ProductFavourite,
    display_config={
        "tag": "Product Favourite",
    },
    serializers={
        "list_serializer": ProductFavouriteSerializer,
        "detail_serializer": ProductFavouriteDetailSerializer,
        "write_serializer": ProductFavouriteWriteSerializer,
    },
)


@extend_schema_view(**schema_config)
class ProductFavouriteViewSet(MultiSerializerMixin, BaseModelViewSet):
    queryset = ProductFavourite.objects.select_related(
        "user", "product"
    ).prefetch_related("product__translations")
    filter_backends = [
        DjangoFilterBackend,
        PascalSnakeCaseOrderingFilter,
        SearchFilter,
    ]
    filterset_fields = ["id", "user_id", "product_id"]
    ordering_fields = [
        "id",
        "user_id",
        "product_id",
        "created_at",
        "updated_at",
    ]
    ordering = ["-created_at"]
    search_fields = [
        "user__username",
        "product__translations__name",
    ]

    serializers = {
        "default": ProductFavouriteDetailSerializer,
        "list": ProductFavouriteSerializer,
        "retrieve": ProductFavouriteDetailSerializer,
        "create": ProductFavouriteWriteSerializer,
        "update": ProductFavouriteWriteSerializer,
        "partial_update": ProductFavouriteWriteSerializer,
        "product": ProductDetailResponseSerializer,
        "favourites_by_products": ProductFavouriteByProductsResponseSerializer,
    }

    def create(self, request, *args, **kwargs):
        return super().create(request, *args, **kwargs)

    def perform_create(self, serializer):
        serializer.save(user=self.request.user)

    @extend_schema(
        operation_id="getProductFavouriteProduct",
        summary=_("Get favourite product details"),
        description=_(
            "Get detailed information about the product in this favourite entry."
        ),
        tags=["Product Favourites"],
        responses={
            200: ProductDetailResponseSerializer,
        },
    )
    @action(detail=True, methods=["GET"])
    def product(self, request, *args, **kwargs):
        product_favourite = self.get_object()
        serializer = self.get_serializer(
            product_favourite.product, context=self.get_serializer_context()
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="getProductFavouritesByProducts",
        summary=_("Get favourites by product IDs"),
        description=_(
            "Get favourite entries for the specified product IDs. Requires authentication."
        ),
        tags=["Product Favourites"],
        request=ProductFavouriteByProductsRequestSerializer,
        responses={
            200: ProductFavouriteByProductsResponseSerializer(many=True),
        },
    )
    @action(detail=False, methods=["POST"])
    def favourites_by_products(self, request, *args, **kwargs):
        user = request.user
        serializer = ProductFavouriteByProductsRequestSerializer(
            data=request.data
        )
        serializer.is_valid(raise_exception=True)

        product_ids = serializer.validated_data["product_ids"]
        favourites = ProductFavourite.objects.filter(
            user=user, product_id__in=product_ids
        ).select_related("user", "product")

        response_serializer = self.get_serializer(favourites, many=True)
        return Response(response_serializer.data, status=status.HTTP_200_OK)



from __future__ import annotations

from django.utils.translation import gettext_lazy as _
from rest_framework import serializers


class UsernameUpdateSerializer(serializers.Serializer):
    username = serializers.CharField(
        max_length=150,
        help_text=_("New username"),
    )


class UsernameUpdateResponseSerializer(serializers.Serializer):
    detail = serializers.CharField(
        help_text=_("Success message for username update")
    )


class UserSubscriptionSummaryResponseSerializer(serializers.Serializer):
    total_subscriptions = serializers.IntegerField()
    active_subscriptions = serializers.IntegerField()
    categories = serializers.ListField(child=serializers.CharField())



import django_filters

from user.models.subscription import SubscriptionTopic, UserSubscription


class SubscriptionTopicFilter(django_filters.FilterSet):
    category = django_filters.ChoiceFilter(
        choices=SubscriptionTopic.TopicCategory.choices
    )
    is_active = django_filters.BooleanFilter()
    is_default = django_filters.BooleanFilter()
    requires_confirmation = django_filters.BooleanFilter()

    slug = django_filters.CharFilter(lookup_expr="icontains")
    slug_exact = django_filters.CharFilter(
        field_name="slug", lookup_expr="exact"
    )

    created_after = django_filters.DateTimeFilter(
        field_name="created_at", lookup_expr="gte"
    )
    created_before = django_filters.DateTimeFilter(
        field_name="created_at", lookup_expr="lte"
    )
    updated_after = django_filters.DateTimeFilter(
        field_name="updated_at", lookup_expr="gte"
    )
    updated_before = django_filters.DateTimeFilter(
        field_name="updated_at", lookup_expr="lte"
    )

    name = django_filters.CharFilter(
        field_name="translations__name", lookup_expr="icontains"
    )
    description = django_filters.CharFilter(
        field_name="translations__description", lookup_expr="icontains"
    )

    class Meta:
        model = SubscriptionTopic
        fields = [
            "category",
            "is_active",
            "is_default",
            "requires_confirmation",
            "slug",
            "slug_exact",
            "created_after",
            "created_before",
            "updated_after",
            "updated_before",
            "name",
            "description",
        ]


class UserSubscriptionFilter(django_filters.FilterSet):
    topic = django_filters.ModelChoiceFilter(
        queryset=SubscriptionTopic.objects.filter(is_active=True)
    )
    status = django_filters.ChoiceFilter(
        choices=UserSubscription.SubscriptionStatus.choices
    )

    topic__category = django_filters.ChoiceFilter(
        field_name="topic__category",
        choices=SubscriptionTopic.TopicCategory.choices,
    )

    subscribed_after = django_filters.DateTimeFilter(
        field_name="subscribed_at", lookup_expr="gte"
    )
    subscribed_before = django_filters.DateTimeFilter(
        field_name="subscribed_at", lookup_expr="lte"
    )
    unsubscribed_after = django_filters.DateTimeFilter(
        field_name="unsubscribed_at", lookup_expr="gte"
    )
    unsubscribed_before = django_filters.DateTimeFilter(
        field_name="unsubscribed_at", lookup_expr="lte"
    )
    created_after = django_filters.DateTimeFilter(
        field_name="created_at", lookup_expr="gte"
    )
    created_before = django_filters.DateTimeFilter(
        field_name="created_at", lookup_expr="lte"
    )
    updated_after = django_filters.DateTimeFilter(
        field_name="updated_at", lookup_expr="gte"
    )
    updated_before = django_filters.DateTimeFilter(
        field_name="updated_at", lookup_expr="lte"
    )

    topic__slug = django_filters.CharFilter(
        field_name="topic__slug", lookup_expr="icontains"
    )
    topic__slug_exact = django_filters.CharFilter(
        field_name="topic__slug", lookup_expr="exact"
    )

    topic_name = django_filters.CharFilter(
        field_name="topic__translations__name", lookup_expr="icontains"
    )
    topic_description = django_filters.CharFilter(
        field_name="topic__translations__description", lookup_expr="icontains"
    )

    class Meta:
        model = UserSubscription
        fields = [
            "topic",
            "status",
            "topic__category",
            "subscribed_after",
            "subscribed_before",
            "unsubscribed_after",
            "unsubscribed_before",
            "created_after",
            "created_before",
            "updated_after",
            "updated_before",
            "topic__slug",
            "topic__slug_exact",
            "topic_name",
            "topic_description",
        ]
